@@ -1,16 +1,18 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/* $Id$ */
+
 /* Default polygon and image drawing device procedures */
 #include "math_.h"
 #include "memory_.h"
@@ -804,11 +806,40 @@ gx_default_draw_thin_line(gx_device * dev,
                 SWAP(fx0, fx1, tf), SWAP(fy0, fy1, tf),
                     h = -h;
             /* So we are plotting a trapezoid with horizontal thin edges.
-             * Check for whether a triangular extension area on the end
-             * covers an additional pixel centre. */
-            {
-                int deltay = int2fixed(fixed2int_var(fy1)) + fixed_half -fy1;
-                int deltax = int2fixed(fixed2int_var(fx1)) + fixed_half -fx1;
+             * If we are drawing a non-axis aligned trap, then we check
+             * for whether a triangular extension area on the end covers an
+             * additional pixel centre; if so, we fill an extra pixel.
+             * If we are drawing an axis aligned trap and fill adjust is 0,
+             * then we shouldn't need to do this.
+             * If we are drawing an axis aligned trap, and fill adjust is non
+             * zero, then perform the check, but with a "butt cap" rather than
+             * a "triangle cap" region.
+             * See bug 687721 and bug 693212 for this history of this.
+             */
+            if (w == 0 && adjusty) {
+                int deltay;
+                deltay = int2fixed(fixed2int_var(fy1)) + fixed_half -fy1;
+
+                if ((deltay > 0) && (deltay <= fixed_half))
+                {
+                    int c = gx_fill_rectangle_device_rop(fixed2int_var(fx1),
+                                                         fixed2int_var(fy1),
+                                                         1,1,pdevc,dev,lop);
+                    if (c < 0) return c;
+                }
+                deltay = int2fixed(fixed2int_var(fy0)) + fixed_half -fy0;
+
+                if ((deltay < 0) && (deltay >= -fixed_half))
+                {
+                    int c = gx_fill_rectangle_device_rop(fixed2int_var(fx0),
+                                                         fixed2int_var(fy0),
+                                                         1,1,pdevc,dev,lop);
+                    if (c < 0) return c;
+                }
+            } else if (w != 0) {
+                int deltax, deltay;
+                deltay = int2fixed(fixed2int_var(fy1)) + fixed_half -fy1;
+                deltax = int2fixed(fixed2int_var(fx1)) + fixed_half -fx1;
 
                 if (deltax < 0) deltax=-deltax;
                 if ((deltay > 0) && (deltay <= fixed_half) &&
@@ -819,10 +850,8 @@ gx_default_draw_thin_line(gx_device * dev,
                                                          1,1,pdevc,dev,lop);
                     if (c < 0) return c;
                 }
-            }
-            {
-                int deltay = int2fixed(fixed2int_var(fy0)) + fixed_half -fy0;
-                int deltax = int2fixed(fixed2int_var(fx0)) + fixed_half -fx0;
+                deltay = int2fixed(fixed2int_var(fy0)) + fixed_half -fy0;
+                deltax = int2fixed(fixed2int_var(fx0)) + fixed_half -fx0;
 
                 if (deltax < 0) deltax=-deltax;
                 if ((deltay < 0) && (deltay >= -fixed_half) &&
@@ -866,9 +895,30 @@ gx_default_draw_thin_line(gx_device * dev,
             /* So we are plotting a trapezoid with vertical thin edges
              * Check for whether a triangular extension area on the end
              * covers an additional pixel centre. */
-            {
-                int deltax = int2fixed(fixed2int_var(fx1)) + fixed_half -fx1;
-                int deltay = int2fixed(fixed2int_var(fy1)) + fixed_half -fy1;
+            if (h == 0 && adjustx) {
+                int deltax;
+                deltax = int2fixed(fixed2int_var(fx1)) + fixed_half -fx1;
+
+                if ((deltax > 0) && (deltax <= fixed_half))
+                {
+                    int c = gx_fill_rectangle_device_rop(fixed2int_var(fx1),
+                                                         fixed2int_var(fy1),
+                                                         1,1,pdevc,dev,lop);
+                    if (c < 0) return c;
+                }
+                deltax = int2fixed(fixed2int_var(fx0)) + fixed_half -fx0;
+
+                if ((deltax < 0) && (deltax >= -fixed_half))
+                {
+                    int c = gx_fill_rectangle_device_rop(fixed2int_var(fx0),
+                                                         fixed2int_var(fy0),
+                                                         1,1,pdevc,dev,lop);
+                    if (c < 0) return c;
+                }
+            } else if (h != 0) {
+                int deltax, deltay;
+                deltax = int2fixed(fixed2int_var(fx1)) + fixed_half -fx1;
+                deltay = int2fixed(fixed2int_var(fy1)) + fixed_half -fy1;
 
                 if (deltay < 0) deltay=-deltay;
                 if ((deltax > 0) && (deltax <= fixed_half) &&
@@ -879,10 +929,8 @@ gx_default_draw_thin_line(gx_device * dev,
                                                          1,1,pdevc,dev,lop);
                     if (c < 0) return c;
                 }
-            }
-            {
-                int deltax = int2fixed(fixed2int_var(fx0)) + fixed_half -fx0;
-                int deltay = int2fixed(fixed2int_var(fy0)) + fixed_half -fy0;
+                deltax = int2fixed(fixed2int_var(fx0)) + fixed_half -fx0;
+                deltay = int2fixed(fixed2int_var(fy0)) + fixed_half -fy0;
 
                 if (deltay < 0) deltay=-deltay;
                 if ((deltax < 0) && (deltax >= -fixed_half) &&
