@@ -21,7 +21,7 @@
 #include "gdevmem.h"		/* private definitions */
 
 /* Define debugging statistics. */
-#ifdef DEBUG
+#if defined(DEBUG) && !defined(GS_THREADSAFE)
 struct stats_mem56_s {
     long
         fill, fwide, fgray[101], fsetc, fcolor[101], fnarrow[5],
@@ -115,7 +115,7 @@ mem_true56_fill_rectangle(gx_device * dev,
      */
     fit_fill_xywh(dev, x, y, w, h);
     INCR(fill);
-#ifdef DEBUG
+#if defined(DEBUG) && !defined(GS_THREADSAFE)
     stats_mem56.ftotal += w;
 #endif
     if (w >= 5) {
@@ -147,7 +147,7 @@ mem_true56_fill_rectangle(gx_device * dev,
                 INCR(fsetc);
                 set_color56_cache(color, a, b, c, d, e, f, g);
             }
-#ifdef DEBUG
+#if defined(DEBUG) && !defined(GS_THREADSAFE)
             {
                 int ci;
                 for (ci = 0; ci < prev_count; ++ci)
