@@ -1,16 +1,18 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/* $Id$ */
+
 /* RasterOp implementation for 8N-bit memory devices */
 #include "memory_.h"
 #include "gx.h"
@@ -229,7 +231,7 @@ df:         return mem_default_strip_copy_rop(dev,
                         rop_body_24(const_source, const_texture);
                     }
                 if (memcmp(testbuffer, start, bytelen) != 0) {
-                    eprintf("Failed!\n");
+                    emprintf(dev->memory, "Failed!\n");
                 }
 #else
                 rop_run(&ropper, drow, width);
@@ -275,7 +277,7 @@ df:         return mem_default_strip_copy_rop(dev,
                             rop_body_24(s_pixel, const_texture);
                         }
                     if (memcmp(testbuffer, start, bytelen) != 0) {
-                        eprintf("Failed!\n");
+                        emprintf(dev->memory, "Failed!\n");
                     }
 #else
 /**** 1-bit source ****/
@@ -327,7 +329,7 @@ df:         return mem_default_strip_copy_rop(dev,
                         }
                     }
                     if (memcmp(testbuffer, start, bytelen) != 0) {
-                        eprintf("Failed!\n");
+                        emprintf(dev->memory, "Failed!\n");
                     }
 #else
 /**** 8-bit source & dest ****/
@@ -384,7 +386,7 @@ df:         return mem_default_strip_copy_rop(dev,
                             rop_body_24(const_source, t_pixel);
                         }
                     if (memcmp(testbuffer, start, bytelen) != 0) {
-                        eprintf("Failed!\n");
+                        emprintf(dev->memory, "Failed!\n");
                     }
 #else
                     rop_run(&ropper, dptr, left);
@@ -435,7 +437,7 @@ df:         return mem_default_strip_copy_rop(dev,
                         }
                     }
                     if (memcmp(testbuffer, start, bytelen) != 0) {
-                        eprintf("Failed!\n");
+                        emprintf(dev->memory, "Failed!\n");
                     }
 #else
 /**** 8-bit T & D ****/
@@ -524,7 +526,7 @@ df:         return mem_default_strip_copy_rop(dev,
                         }
                     }
                     if (memcmp(testbuffer, start, bytelen) != 0) {
-                        eprintf("Failed!\n");
+                        emprintf(dev->memory, "Failed!\n");
                     }
 #else
                     rop_run(&ropper, dptr, left);
@@ -574,7 +576,7 @@ df:         return mem_default_strip_copy_rop(dev,
                         }
                     }
                     if (memcmp(testbuffer, start, bytelen) != 0) {
-                        eprintf("Failed!\n");
+                        emprintf(dev->memory, "Failed!\n");
                     }
 #else
 /**** 8-bit destination ****/
