@@ -18,7 +18,9 @@
 #include "ctype_.h"
 #include "gdevprn.h"
 #include "gp.h"
+#include "gdevdevn.h"           /* for gs_devn_params_s */
 #include "gsdevice.h"		/* for gs_deviceinitialmatrix */
+#include "gxdevsop.h"		/* for gxdso_* */
 #include "gsfname.h"
 #include "gsparam.h"
 #include "gxclio.h"
@@ -30,7 +32,6 @@
 /*#define DEBUGGING_HACKS*/
 
 /* GC information */
-#define PRINTER_IS_CLIST(pdev) ((pdev)->buffer_space != 0)
 static
 ENUM_PTRS_WITH(device_printer_enum_ptrs, gx_device_printer *pdev)
     if (PRINTER_IS_CLIST(pdev))
@@ -53,6 +54,8 @@ public_st_device_printer();
 /* Define the standard printer procedure vector. */
 const gx_device_procs prn_std_procs =
     prn_procs(gdev_prn_open, gdev_prn_output_page, gdev_prn_close);
+const gx_device_procs prn_bg_procs =
+    prn_procs(gdev_prn_open, gdev_prn_bg_output_page, gdev_prn_close);
 
 /* Forward references */
 int gdev_prn_maybe_realloc_memory(gx_device_printer *pdev,
@@ -60,7 +63,17 @@ int gdev_prn_maybe_realloc_memory(gx_device_printer *pdev,
                                   int old_width, int old_height,
                                   bool old_page_uses_transparency);
 
+static int
+gdev_prn_output_page_aux(gx_device * pdev, int num_copies, int flush, bool seekable, bool bg_print_ok);
+
 extern dev_proc_open_device(pattern_clist_open_device);
+extern dev_proc_open_device(clist_open);
+
+/* The function run in a background thread */
+static void prn_print_page_in_background(void *data);
+
+/* wait for a background thread to finish and clean up background printing */
+static void prn_finish_bg_print(gx_device_printer *ppdev);
 
 /* ------ Open/close ------ */
 
@@ -81,6 +94,25 @@ gdev_prn_open(gx_device * pdev)
     return code;
 }
 
+/* This is called various places to wait for any pending bg print thread and */
+/* perform its cleanup                                                       */
+static void
+prn_finish_bg_print(gx_device_printer *ppdev)
+{
+    /* if we have a a bg printing device that was created, then wait for its	*/
+    /* semaphore (it may already have been signalled, but that's OK.) then	*/
+    /* close and unlink the files and free the device and its private allocator	*/
+    if (ppdev->bg_print.device != NULL) {
+        int closecode;
+        gx_semaphore_wait(ppdev->bg_print.sema);
+        closecode = gdev_prn_close_printer((gx_device *)ppdev);
+        if (ppdev->bg_print.return_code == 0)
+            ppdev->bg_print.return_code = closecode;	/* return code here iff there wasn't another error */
+        teardown_device_and_mem_for_thread(ppdev->bg_print.device,
+                                           ppdev->bg_print.thread_id, true);
+        ppdev->bg_print.device = NULL;
+    }
+}
 /* Generic closing for the printer device. */
 /* Specific devices may wish to extend this. */
 int
@@ -89,14 +121,42 @@ gdev_prn_close(gx_device * pdev)
     gx_device_printer * const ppdev = (gx_device_printer *)pdev;
     int code = 0;
 
+    prn_finish_bg_print(ppdev);
+    if (ppdev->bg_print.sema != NULL) {
+        gx_semaphore_free(ppdev->bg_print.sema);
+        ppdev->bg_print.sema = NULL;		/* prevent double free */
+    }
     gdev_prn_free_memory(pdev);
     if (ppdev->file != NULL) {
         code = gx_device_close_output_file(pdev, ppdev->fname, ppdev->file);
         ppdev->file = NULL;
     }
+    if (ppdev->saved_pages_list != NULL) {
+        gx_saved_pages_list_free(ppdev->saved_pages_list);
+        ppdev->saved_pages_list = NULL;
+    }
     return code;
 }
 
+int
+gdev_prn_forwarding_dev_spec_op(gx_device *pdev, int dev_spec_op, void *data, int size)
+{
+    gx_device_printer *ppdev = (gx_device_printer *)pdev;
+
+    if (ppdev->orig_procs.dev_spec_op)
+        return ppdev->orig_procs.dev_spec_op(pdev, dev_spec_op, data, size);
+    return gdev_prn_dev_spec_op(pdev, dev_spec_op, data, size);
+}
+
+int
+gdev_prn_dev_spec_op(gx_device *pdev, int dev_spec_op, void *data, int size)
+{
+    if (dev_spec_op == gxdso_supports_saved_pages)
+        return 1;
+
+    return gx_default_dev_spec_op(pdev, dev_spec_op, data, size);
+}
+
 static int		/* returns 0 ok, else -ve error cde */
 gdev_prn_setup_as_command_list(gx_device *pdev, gs_memory_t *buffer_memory,
                                byte **the_memory,
@@ -117,9 +177,9 @@ gdev_prn_setup_as_command_list(gx_device *pdev, gs_memory_t *buffer_memory,
   BEGIN\
     ulong *fp_ = (ulong *)&first_arg - 2;\
     for (; fp_ && (fp_[1] & 0xff000000) == 0x08000000; fp_ = (ulong *)*fp_)\
-        dprintf2("  fp=0x%lx ip=0x%lx\n", (ulong)fp_, fp_[1]);\
+        dmprintf2(buffer_memory, "  fp=0x%lx ip=0x%lx\n", (ulong)fp_, fp_[1]);\
   END
-dputs("alloc buffer:\n");
+dmputs(buffer_memory, "alloc buffer:\n");
 BACKTRACE(pdev);
 #endif /*DEBUGGING_HACKS*/
     for ( space = space_params->BufferSpace; ; ) {
@@ -142,10 +202,12 @@ BACKTRACE(pdev);
 open_c:
     ppdev->buf = base;
     ppdev->buffer_space = space;
+    pclist_dev->common.is_printer = 1;
     clist_init_io_procs(pclist_dev, ppdev->BLS_force_memory);
     clist_init_params(pclist_dev, base, space, pdev,
                       ppdev->printer_procs.buf_procs,
-                      space_params->band, ppdev->is_async_renderer,
+                      space_params->band,
+                      false, /* do_not_open_or_close_bandfiles */
                       (ppdev->bandlist_memory == 0 ? pdev->memory->non_gc_memory:
                        ppdev->bandlist_memory),
                       ppdev->free_up_bandlist_memory,
@@ -194,27 +256,28 @@ gdev_prn_tear_down(gx_device *pdev, byte **the_memory)
     gx_device_memory * const pmemdev = (gx_device_memory *)pdev;
     gx_device_clist *const pclist_dev = (gx_device_clist *)pdev;
     gx_device_clist_common * const pcldev = &pclist_dev->common;
-    bool is_command_list;
+    gx_device_clist_reader * const pcrdev = &pclist_dev->reader;
+    bool was_command_list;
 
-    if (ppdev->buffer_space != 0) {
+    if (PRINTER_IS_CLIST(ppdev)) {
         /* Close cmd list device & point to the storage */
         (*gs_clist_device_procs.close_device)( (gx_device *)pcldev );
         *the_memory = ppdev->buf;
         ppdev->buf = 0;
         ppdev->buffer_space = 0;
-        is_command_list = true;
+        was_command_list = true;
 
-        /* If the clist is a reader clist, free any band_complexity_array
+        /* If the clist is a reader clist, free any color_usage_array
          * memory used by same.
          */
         if (!CLIST_IS_WRITER(pclist_dev))
-            gx_clist_reader_free_band_complexity_array(pclist_dev);
+            gs_free_object(pcrdev->memory, pcrdev->color_usage_array, "clist_color_usage_array");
 
     } else {
         /* point at the device bitmap, no need to close mem dev */
         *the_memory = pmemdev->base;
         pmemdev->base = 0;
-        is_command_list = false;
+        was_command_list = false;
     }
 
     /* Reset device proc vector to default */
@@ -222,7 +285,7 @@ gdev_prn_tear_down(gx_device *pdev, byte **the_memory)
         pdev->procs = ppdev->orig_procs;
     ppdev->orig_procs.open_device = 0;	/* prevent uninit'd restore of procs */
 
-    return is_command_list;
+    return was_command_list;
 }
 
 static int
@@ -313,6 +376,7 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
             is_command_list = save_is_command_list;
         else {
             is_command_list = space_params.banding_type == BandingAlways ||
+                ppdev->saved_pages_list != NULL ||
                 mem_space >= space_params.MaxBitmap ||
                 !size_ok;	    /* too big to allocate */
         }
@@ -386,7 +450,6 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
                 ppdev->orig_procs.open_device = 0;	/* prevent uninit'd restore of procs */
                 return_error(code);
             }
-            pmemdev->base = base;
         }
         if (ecode == 0)
             break;
@@ -501,20 +564,16 @@ gdev_prn_get_params(gx_device * pdev, gs_param_list * plist)
     int code = gx_default_get_params(pdev, plist);
     gs_param_string ofns;
     gs_param_string bls;
+    gs_param_string saved_pages;
 
     if (code < 0 ||
-        (code = param_write_long(plist, "BandBufferSpace", &ppdev->space_params.band.BandBufferSpace)) < 0 ||
-        (code = param_write_int(plist, "BandHeight", &ppdev->space_params.band.BandHeight)) < 0 ||
-        (code = param_write_int(plist, "BandWidth", &ppdev->space_params.band.BandWidth)) < 0 ||
-        (code = param_write_long(plist, "BufferSpace", &ppdev->space_params.BufferSpace)) < 0 ||
         (ppdev->Duplex_set >= 0 &&
         (code = (ppdev->Duplex_set ?
                   param_write_bool(plist, "Duplex", &ppdev->Duplex) :
                   param_write_null(plist, "Duplex"))) < 0) ||
-        (code = param_write_long(plist, "MaxBitmap", &ppdev->space_params.MaxBitmap)) < 0 ||
         (code = param_write_int(plist, "NumRenderingThreads", &ppdev->num_render_threads_requested)) < 0 ||
         (code = param_write_bool(plist, "OpenOutputFile", &ppdev->OpenOutputFile)) < 0 ||
-        (code = param_write_bool(plist, "PageUsesTransparency", &ppdev->page_uses_transparency)) < 0 ||
+        (code = param_write_bool(plist, "BGPrint", &ppdev->bg_print_requested)) < 0 ||
         (code = param_write_bool(plist, "ReopenPerPage", &ppdev->ReopenPerPage)) < 0
         )
         return code;
@@ -537,7 +596,15 @@ gdev_prn_get_params(gx_device * pdev, gs_param_list * plist)
     ofns.data = (const byte *)ppdev->fname,
         ofns.size = strlen(ppdev->fname),
         ofns.persistent = false;
-    return param_write_string(plist, "OutputFile", &ofns);
+    if ((code = param_write_string(plist, "OutputFile", &ofns)) < 0)
+        return code;
+
+    /* Always return an empty string for saved-pages so that get_params followed */
+    /* by put_params will have no effect.                                       */
+    saved_pages.data = (const byte *)"";
+    saved_pages.size = 0;
+    saved_pages.persistent = false;
+    return param_write_string(plist, "saved-pages", &saved_pages);
 }
 
 /* Validate an OutputFile name by checking any %-formats. */
@@ -562,20 +629,21 @@ gdev_prn_put_params(gx_device * pdev, gs_param_list * plist)
     bool is_open = pdev->is_open;
     bool oof = ppdev->OpenOutputFile;
     bool rpp = ppdev->ReopenPerPage;
-    bool page_uses_transparency = ppdev->page_uses_transparency;
     bool old_page_uses_transparency = ppdev->page_uses_transparency;
+    bool bg_print_requested = ppdev->bg_print_requested;
     bool duplex;
     int duplex_set = -1;
     int width = pdev->width;
     int height = pdev->height;
     int nthreads = ppdev->num_render_threads_requested;
-    gdev_prn_space_params sp, save_sp;
+    gdev_prn_space_params save_sp;
     gs_param_string ofs;
     gs_param_string bls;
     gs_param_dict mdict;
+    gs_param_string saved_pages;
 
-    sp = ppdev->space_params;
-    save_sp = sp;
+    memset(&saved_pages, 0, sizeof(gs_param_string));
+    save_sp = ppdev->space_params;
 
     switch (code = param_read_bool(plist, (param_name = "OpenOutputFile"), &oof)) {
         default:
@@ -595,16 +663,6 @@ gdev_prn_put_params(gx_device * pdev, gs_param_list * plist)
             break;
     }
 
-    switch (code = param_read_bool(plist, (param_name = "PageUsesTransparency"),
-                                                        &page_uses_transparency)) {
-        default:
-            ecode = code;
-            param_signal_error(plist, param_name, ecode);
-        case 0:
-        case 1:
-            break;
-    }
-
     if (ppdev->Duplex_set >= 0)	/* i.e., Duplex is supported */
         switch (code = param_read_bool(plist, (param_name = "Duplex"),
                                        &duplex)) {
@@ -621,40 +679,6 @@ gdev_prn_put_params(gx_device * pdev, gs_param_list * plist)
             case 1:
                 ;
         }
-#define CHECK_PARAM_CASES(member, bad, label)\
-    case 0:\
-        if ((sp.params_are_read_only ? sp.member != save_sp.member : bad))\
-            ecode = gs_error_rangecheck;\
-        else\
-            break;\
-        goto label;\
-    default:\
-        ecode = code;\
-label:\
-        param_signal_error(plist, param_name, ecode);\
-    case 1:\
-        break
-
-    switch (code = param_read_long(plist, (param_name = "MaxBitmap"), &sp.MaxBitmap)) {
-        CHECK_PARAM_CASES(MaxBitmap, sp.MaxBitmap < 0, mbe);
-    }
-
-    switch (code = param_read_long(plist, (param_name = "BufferSpace"), &sp.BufferSpace)) {
-        CHECK_PARAM_CASES(BufferSpace, sp.BufferSpace < 10000, bse);
-    }
-
-    switch (code = param_read_int(plist, (param_name = "BandWidth"), &sp.band.BandWidth)) {
-        CHECK_PARAM_CASES(band.BandWidth, sp.band.BandWidth < 0, bwe);
-    }
-
-    switch (code = param_read_int(plist, (param_name = "BandHeight"), &sp.band.BandHeight)) {
-        CHECK_PARAM_CASES(band.BandHeight, sp.band.BandHeight < 0, bhe);
-    }
-
-    switch (code = param_read_long(plist, (param_name = "BandBufferSpace"), &sp.band.BandBufferSpace)) {
-        CHECK_PARAM_CASES(band.BandBufferSpace, sp.band.BandBufferSpace < 0, bbse);
-    }
-
     switch (code = param_read_string(plist, (param_name = "BandListStorage"), &bls)) {
         case 0:
             /* Only accept 'file' if the file procs are include in the build */
@@ -717,6 +741,26 @@ label:\
         case 1:
             ;
     }
+    switch (code = param_read_bool(plist, (param_name = "BGPrint"),
+                                                        &bg_print_requested)) {
+        default:
+            ecode = code;
+            param_signal_error(plist, param_name, ecode);
+        case 0:
+        case 1:
+            break;
+    }
+
+    switch (code = param_read_string(plist, (param_name = "saved-pages"),
+                                                        &saved_pages)) {
+        default:
+            ecode = code;
+            param_signal_error(plist, param_name, ecode);
+        case 0:
+        case 1:
+            break;
+    }
+
 
     if (ecode < 0)
         return ecode;
@@ -729,12 +773,11 @@ label:\
 
     ppdev->OpenOutputFile = oof;
     ppdev->ReopenPerPage = rpp;
-    ppdev->page_uses_transparency = page_uses_transparency;
+    ppdev->bg_print_requested = bg_print_requested;
     if (duplex_set >= 0) {
         ppdev->Duplex = duplex;
         ppdev->Duplex_set = duplex_set;
     }
-    ppdev->space_params = sp;
     ppdev->num_render_threads_requested = nthreads;
     if (bls.data != 0) {
         ppdev->BLS_force_memory = (bls.data[0] == 'm');
@@ -744,7 +787,7 @@ label:\
     /* Formerly, would not reallocate if device is not open: */
     /* we had to patch this out (see News for 5.50). */
     code = gdev_prn_maybe_realloc_memory(ppdev, &save_sp, width, height,
-                                                old_page_uses_transparency);
+                                         old_page_uses_transparency);
     if (code < 0)
         return code;
 
@@ -754,8 +797,9 @@ label:\
                       (const byte *)ppdev->fname, strlen(ppdev->fname))
         ) {
         /* Close the file if it's open. */
-        if (ppdev->file != NULL)
+        if (ppdev->file != NULL) {
             gx_device_close_output_file(pdev, ppdev->fname, ppdev->file);
+        }
         ppdev->file = NULL;
         if (sizeof(ppdev->fname) <= ofs.size)
             return_error(gs_error_limitcheck);
@@ -770,6 +814,13 @@ label:\
         if (code < 0)
             return code;
     }
+
+    /* Processing the saved_pages string MAY have side effects, such as printing, */
+    /* allocating or freeing a list. This is (sort of) a write-only parameter, so */
+    /* the get_params will always return an empty string (a no-op action).        */
+    if (saved_pages.data != 0 && saved_pages.size != 0) {
+        return gx_saved_pages_param_process(ppdev, (byte *)saved_pages.data, saved_pages.size);
+    }
     return 0;
 }
 
@@ -783,41 +834,131 @@ gx_default_get_space_params(const gx_device_printer *printer_dev,
     return;
 }
 
-/* Generic routine to send the page to the printer. */
-int	/* 0 ok, -ve error, or 1 if successfully upgraded to buffer_page */
-gdev_prn_output_page(gx_device * pdev, int num_copies, int flush)
+/* Common routine to send the page to the printer.                               */
+/* If seekable is true, then the printer outputfile must be seekable.            */
+/* If bg_print_ok is true, the device print_page_copies is compatible with the   */
+/* background printing, i.e., thread safe and does not change the device.        */
+/* If the printer device is in 'saved_pages' mode, then background printing is   */
+/* irrelevant and is ignored. In this case, pages are saved to the list.         */
+static int	/* 0 ok, -ve error, or 1 if successfully upgraded to buffer_page */
+gdev_prn_output_page_aux(gx_device * pdev, int num_copies, int flush, bool seekable, bool bg_print_ok)
 {
     gx_device_printer * const ppdev = (gx_device_printer *)pdev;
+    gs_devn_params *pdevn_params;
     int outcode = 0, closecode = 0, errcode = 0, endcode;
     bool upgraded_copypage = false;
+    int code;
 
-    if (num_copies > 0 || !flush) {
-        int code = gdev_prn_open_printer(pdev, 1);
+    prn_finish_bg_print(ppdev);		/* finish any previous background printing */
 
-        if (code < 0)
+    if (num_copies > 0 && ppdev->saved_pages_list != NULL) {
+        /* We are putting pages on a list */
+        if ((code = gx_saved_pages_list_add(ppdev)) < 0)
+            return code;
+
+    } else if (num_copies > 0 || !flush) {
+        if ((code = gdev_prn_open_printer_seekable(pdev, 1, seekable)) < 0)
             return code;
 
         /* If copypage request, try to do it using buffer_page */
         if ( !flush &&
-             (*ppdev->printer_procs.buffer_page)
-             (ppdev, ppdev->file, num_copies) >= 0
-             ) {
+             (*ppdev->printer_procs.buffer_page)(ppdev, ppdev->file, num_copies) >= 0) {
             upgraded_copypage = true;
             flush = true;
-        }
-        else if (num_copies > 0)
-            /* Print the accumulated page description. */
-            outcode =
-                (*ppdev->printer_procs.print_page_copies)(ppdev, ppdev->file,
+
+        } else if (num_copies > 0) {
+            int threads_enabled = 0;
+            int print_foreground = 1;		/* default to foreground printing */
+
+            if (bg_print_ok && PRINTER_IS_CLIST(ppdev) &&
+                (ppdev->bg_print_requested || ppdev->num_render_threads_requested > 0)) {
+                threads_enabled = clist_enable_multi_thread_render(pdev);
+            }
+            /* NB: we leave the semaphore allocated until foreground printing or close */
+            /* If there was an error, abort on this page -- no good way to handle this */
+            /* but it means that the error will be reported AFTER another page was     */
+            /* interpreted and written to clist files. FIXME: ???                      */
+            if (ppdev->bg_print.return_code < 0) {
+                outcode = ppdev->bg_print.return_code;
+                threads_enabled = 0;	/* and allow current page to try foreground */
+            }
+            /* Use 'while' instead of 'if' to avoid nesting */
+            while (ppdev->bg_print_requested && threads_enabled) {
+                gx_device *ndev;
+                gx_device_printer *npdev;
+                gx_device_clist_reader *crdev = (gx_device_clist_reader *)ppdev;
+
+                if ((code = clist_close_writer_and_init_reader((gx_device_clist *)ppdev)) < 0)
+                    /* should not happen -- do foreground print */
+                    break;
+
+                if (ppdev->bg_print.sema == NULL)
+                    if (((ppdev->bg_print.sema = gx_semaphore_alloc(ppdev->memory->non_gc_memory)) == NULL))
+                        break;			/* couldn't create the semaphore */
+
+                ndev = setup_device_and_mem_for_thread(pdev->memory->thread_safe_memory, pdev, true, NULL);
+                if (ndev == NULL) {
+                    break;
+                }
+                ppdev->bg_print.device = ndev;
+                ppdev->bg_print.num_copies = num_copies;
+                npdev = (gx_device_printer *)ndev;
+                npdev->bg_print_requested = 0;
+                npdev->num_render_threads_requested = ppdev->num_render_threads_requested;
+
+                /* Now start the thread to print the page */
+                if ((code == gp_thread_start(prn_print_page_in_background,
+                                             (void *)&(ppdev->bg_print),
+                                             &(ppdev->bg_print.thread_id))) < 0) {
+                    /* Did not start cleanly - clean up is in print_foreground block below */
+                    break;
+                }
+                /* Page was succesfully started in bg_print mode */
+                print_foreground = 0;
+                /* Now we need to set up the next page so it will use new clist files */
+                /* Close (but don't delete) the files since the background printing now owns them */
+                if ((code = crdev->page_info.io_procs->fclose(crdev->page_info.cfile, crdev->page_info.cfname, false)) < 0 ||
+                    (code = crdev->page_info.io_procs->fclose(crdev->page_info.bfile, crdev->page_info.bfname, false)) < 0) {
+                    return_error(gs_error_unknownerror); /* shouldn't happen */
+                }
+                crdev->page_info.cfile = crdev->page_info.bfile = NULL;
+                if ((code = clist_open(pdev)) < 0) 	/* this should do it */
+                    /* OOPS! can't proceed with the next page */
+                    return code;	/* probably ioerror */
+                break;				/* exit the while loop */
+            }
+            if (print_foreground) {
+                /* either bg_print was not requested or was not able to start */
+                if (ppdev->bg_print.sema != NULL && ppdev->bg_print.device != NULL) {
+                    /* There was a problem. Teardown the device and its allocator, but */
+                    /* leave the semaphore for possible later use.                     */
+                    teardown_device_and_mem_for_thread(ppdev->bg_print.device,
+                                                       ppdev->bg_print.thread_id, true);
+                    ppdev->bg_print.device = NULL;
+                }
+                /* Here's where we actually let the device's print_page_copies work */
+                /* Print the accumulated page description. */
+                outcode = (*ppdev->printer_procs.print_page_copies)(ppdev, ppdev->file,
                                                           num_copies);
-        fflush(ppdev->file);
-        errcode =
-            (ferror(ppdev->file) ? gs_note_error(gs_error_ioerror) : 0);
-        if (!upgraded_copypage)
-            closecode = gdev_prn_close_printer(pdev);
+                fflush(ppdev->file);
+                errcode = (ferror(ppdev->file) ? gs_note_error(gs_error_ioerror) : 0);
+                /* NB: background printing does this differently in its thread */
+                if (!upgraded_copypage)
+                    closecode = gdev_prn_close_printer(pdev);
+            }
+        }
     }
-    endcode = (ppdev->buffer_space && !ppdev->is_async_renderer ?
-               clist_finish_page(pdev, flush) : 0);
+    /* In the case of a separation device, we need to make sure we */
+    /* clear the separation info before the next page starts.      */
+    pdevn_params = dev_proc(pdev, ret_devn_params)(pdev);
+    if (pdevn_params != NULL) {
+        /* Free up the list of spot names as they were only relevent to that page */
+        free_separation_names(pdev->memory, &(pdevn_params->separations));
+        pdevn_params->num_separation_order_names = 0;
+    }
+    endcode = (PRINTER_IS_CLIST(ppdev) &&
+              !((gx_device_clist_common *)ppdev)->do_not_open_or_close_bandfiles ?
+              clist_finish_page(pdev, flush) : 0);
 
     if (outcode < 0)
         return outcode;
@@ -831,6 +972,30 @@ gdev_prn_output_page(gx_device * pdev, int num_copies, int flush)
     return (endcode < 0 ? endcode : upgraded_copypage ? 1 : 0);
 }
 
+int
+gdev_prn_output_page(gx_device * pdev, int num_copies, int flush)
+{
+    return(gdev_prn_output_page_aux(pdev, num_copies, flush, false, false));
+}
+
+int
+gdev_prn_output_page_seekable(gx_device * pdev, int num_copies, int flush)
+{
+    return(gdev_prn_output_page_aux(pdev, num_copies, flush, true, false));
+}
+
+int
+gdev_prn_bg_output_page(gx_device * pdev, int num_copies, int flush)
+{
+    return(gdev_prn_output_page_aux(pdev, num_copies, flush, false, true));
+}
+
+int
+gdev_prn_bg_output_page_seekable(gx_device * pdev, int num_copies, int flush)
+{
+    return(gdev_prn_output_page_aux(pdev, num_copies, flush, true, true));
+}
+
 /* Print a single copy of a page by calling print_page_copies. */
 int
 gx_print_page_single_copy(gx_device_printer * pdev, FILE * prn_stream)
@@ -887,6 +1052,29 @@ gx_default_buffer_page(gx_device_printer *pdev, FILE *prn_stream,
     return gs_error_unknownerror;
 }
 
+/*
+ * Print a page in the background. When printing is complete,
+ * post the return code and signal the foreground (semaphore).
+ * This is the procedure that is run in the background thread.
+ */
+static void
+prn_print_page_in_background(void *data)
+{
+    bg_print_t *bg_print = (bg_print_t *)data;
+    int code, errcode = 0;
+    int num_copies = bg_print->num_copies;
+    gx_device_printer *ppdev = (gx_device_printer *)bg_print->device;
+
+    code = (*ppdev->printer_procs.print_page_copies)(ppdev, ppdev->file,
+                                                          num_copies);
+    fflush(ppdev->file);
+
+    errcode = (ferror(ppdev->file) ? gs_note_error(gs_error_ioerror) : 0);
+    bg_print->return_code = code < 0 ? code : errcode;
+
+    /* Finally, release the foreground that may be waiting */
+    gx_semaphore_signal(bg_print->sema);
+}
 /* ---------------- Driver services ---------------- */
 
 /* Initialize a rendering plane specification. */
@@ -961,6 +1149,20 @@ gdev_prn_open_printer_seekable(gx_device *pdev, bool binary_mode,
                                               &ppdev->file);
         if (code < 0)
             return code;
+
+        if (seekable && !gp_fseekable(ppdev->file)) {
+            errprintf(pdev->memory, "I/O Error: Output File \"%s\" must be seekable\n", ppdev->fname);
+            if (!IS_LIBCTX_STDOUT(pdev->memory, ppdev->file)
+              && !IS_LIBCTX_STDERR(pdev->memory ,ppdev->file)) {
+
+                code = gx_device_close_output_file(pdev, ppdev->fname, ppdev->file);
+                if (code < 0)
+                    return code;
+            }
+            ppdev->file = NULL;
+
+            return_error(gs_error_ioerror);
+        }
     }
     ppdev->file_is_new = true;
     return 0;
@@ -983,47 +1185,60 @@ gdev_prn_file_is_new(const gx_device_printer *pdev)
 }
 
 /* Determine the colors used in a range of lines. */
+/* FIXME: Currently, the page_info is ignored and the page_info from
+ * the 'dev' parameter is used. For saved pages, the 'dev' may not
+ * be a clist and the page_info should be used for clist file info
+ * which may require a dummy gx_device_printer with the page_info
+ * copied from the caller's data.
+ */
 int
 gx_page_info_color_usage(const gx_device *dev,
                          const gx_band_page_info_t *page_info,
                          int y, int height,
                          gx_color_usage_t *color_usage, int *range_start)
 {
+    gx_device_clist_reader *crdev = (gx_device_clist_reader *)dev;
     int start, end, i;
-    int num_lines = page_info->scan_lines_per_colors_used;
+    int band_height = page_info->band_params.BandHeight;
     gx_color_usage_bits or = 0;
     bool slow_rop = false;
 
     if (y < 0 || height < 0 || height > dev->height - y)
         return -1;
-    start = y / num_lines;
-    end = (y + height + num_lines - 1) / num_lines;
+    start = y / band_height;
+    end = (y + height + band_height - 1) / band_height;
+    if (crdev->color_usage_array == NULL) {
+    }
     for (i = start; i < end; ++i) {
-        or |= page_info->band_color_usage[i].or;
-        slow_rop |= page_info->band_color_usage[i].slow_rop;
+        or |= crdev->color_usage_array[i].or;
+        slow_rop |= crdev->color_usage_array[i].slow_rop;
     }
     color_usage->or = or;
     color_usage->slow_rop = slow_rop;
-    *range_start = start * num_lines;
-    return min(end * num_lines, dev->height) - *range_start;
+    *range_start = start * band_height;
+    return min(end * band_height, dev->height) - *range_start;
 }
 int
 gdev_prn_color_usage(gx_device *dev, int y, int height,
                      gx_color_usage_t *color_usage, int *range_start)
 {
-    gx_device_clist_writer *cldev;
+    gx_device_printer *pdev = (gx_device_printer *)dev;
+    gx_device_clist *cdev = (gx_device_clist *)dev;
+    gx_device_clist_writer *cldev = (gx_device_clist_writer *)dev;
 
     /* If this isn't a banded device, return default values. */
-    if (dev_proc(dev, open_device) != gs_clist_device_procs.open_device) {
+    if (!PRINTER_IS_CLIST(pdev)) {
         *range_start = 0;
         color_usage->or = gx_color_usage_all(dev);
         return dev->height;
     }
-    cldev = (gx_device_clist_writer *)dev;
-    if (cldev->page_info.scan_lines_per_colors_used == 0) /* not set yet */
-        clist_compute_color_usage(cldev);
-    return
-        gx_page_info_color_usage(dev, &cldev->page_info,
+    if (y < 0 || height < 0 || height > dev->height - y)
+        return -1;
+    if (CLIST_IS_WRITER(cdev)) {
+        /* Not expected to be used since usually this is called during reading */
+        return clist_writer_color_usage(cldev, y, height, color_usage, range_start);
+    } else
+        return gx_page_info_color_usage(dev, &cldev->page_info,
                                  y, height, color_usage, range_start);
 }
 
@@ -1035,9 +1250,9 @@ int
 gdev_create_buf_device(create_buf_device_proc_t cbd_proc, gx_device **pbdev,
                        gx_device *target, int y,
                        const gx_render_plane_t *render_plane,
-                       gs_memory_t *mem, gx_band_complexity_t *band_complexity)
+                       gs_memory_t *mem, gx_color_usage_t *color_usage)
 {
-    int code = cbd_proc(pbdev, target, y, render_plane, mem, band_complexity);
+    int code = cbd_proc(pbdev, target, y, render_plane, mem, color_usage);
 
     if (code < 0)
         return code;
@@ -1052,7 +1267,7 @@ gdev_create_buf_device(create_buf_device_proc_t cbd_proc, gx_device **pbdev,
  */
 int
 gx_default_create_buf_device(gx_device **pbdev, gx_device *target, int y,
-    const gx_render_plane_t *render_plane, gs_memory_t *mem, gx_band_complexity_t *band_complexity)
+    const gx_render_plane_t *render_plane, gs_memory_t *mem, gx_color_usage_t *color_usage)
 {
     int plane_index = (render_plane ? render_plane->index : -1);
     int depth;
@@ -1062,8 +1277,12 @@ gx_default_create_buf_device(gx_device **pbdev, gx_device *target, int y,
 
     if (plane_index >= 0)
         depth = render_plane->depth;
-    else
+    else {
         depth = target->color_info.depth;
+        if (target->is_planar)
+            depth /= target->color_info.num_components;
+    }
+
     mdproto = gdev_mem_device_for_bits(depth);
     if (mdproto == 0)
         return_error(gs_error_rangecheck);
@@ -1079,13 +1298,25 @@ gx_default_create_buf_device(gx_device **pbdev, gx_device *target, int y,
         /* The following is a special hack for setting up printer devices. */
         assign_dev_procs(mdev, mdproto);
         check_device_separable((gx_device *)mdev);
+        /* In order for saved-pages to work, we need to hook the dev_spec_op */
+        if (mdev->procs.dev_spec_op == NULL)
+            set_dev_proc(mdev, dev_spec_op, gdev_prn_dev_spec_op);
+#ifdef DEBUG
+        /* scanning sources didn't show anything, but if a device gets changed or added */
+        /* that has its own dev_spec_op, it should call the gdev_prn_spec_op as well    */
+        else
+            errprintf(mdev->memory, "Warning: printer device has private dev_spec_op\n");
+#endif
         gx_device_fill_in_procs((gx_device *)mdev);
     } else {
-        gs_make_mem_device(mdev, mdproto, mem, (band_complexity == NULL ? 1 : 0),
-                           (target == (gx_device *)mdev ? NULL : target));
+        gs_make_mem_device(mdev, mdproto, mem, (color_usage == NULL ? 1 : 0),
+                           target);
     }
     mdev->width = target->width;
     mdev->band_y = y;
+    mdev->log2_align_mod = target->log2_align_mod;
+    mdev->pad = target->pad;
+    mdev->is_planar = target->is_planar;
     /*
      * The matrix in the memory device is irrelevant,
      * because all we do with the device is call the device-level
@@ -1130,7 +1361,9 @@ gx_default_size_buf_device(gx_device_buf_space_t *space, gx_device *target,
         (render_plane && render_plane->index >= 0 ? render_plane->depth :
          target->color_info.depth);
     mdev.width = target->width;
-    mdev.num_planes = 0;
+    mdev.is_planar = target->is_planar;
+    mdev.pad = target->pad;
+    mdev.log2_align_mod = target->log2_align_mod;
     if (gdev_mem_bits_size(&mdev, target->width, height, &(space->bits)) < 0)
         return_error(gs_error_VMerror);
     space->line_ptrs = gdev_mem_line_ptrs_size(&mdev, target->width, height);
@@ -1140,7 +1373,7 @@ gx_default_size_buf_device(gx_device_buf_space_t *space, gx_device *target,
 
 /* Set up the buffer device. */
 int
-gx_default_setup_buf_device(gx_device *bdev, byte *buffer, int bytes_per_line,
+gx_default_setup_buf_device(gx_device *bdev, byte *buffer, int raster,
                             byte **line_ptrs, int y, int setup_height,
                             int full_height)
 {
@@ -1148,12 +1381,8 @@ gx_default_setup_buf_device(gx_device *bdev, byte *buffer, int bytes_per_line,
         (gs_device_is_memory(bdev) ? (gx_device_memory *)bdev :
          (gx_device_memory *)(((gx_device_plane_extract *)bdev)->plane_dev));
     byte **ptrs = line_ptrs;
-    int raster = bytes_per_line;
     int code;
 
-    /****** HACK ******/
-    if ((gx_device *)mdev == bdev && mdev->num_planes)
-        raster = bitmap_raster(mdev->planes[0].depth * mdev->width);
     if (ptrs == 0) {
         /*
          * Before allocating a new line pointer array, if there is a previous
@@ -1169,8 +1398,8 @@ gx_default_setup_buf_device(gx_device *bdev, byte *buffer, int bytes_per_line,
          */
         ptrs = (byte **)
             gs_alloc_byte_array(mdev->memory,
-                                (mdev->num_planes ?
-                                 full_height * mdev->num_planes :
+                                (mdev->is_planar ?
+                                 full_height * mdev->color_info.num_components :
                                  setup_height),
                                 sizeof(byte *), "setup_buf_device");
         if (ptrs == 0)
@@ -1179,7 +1408,7 @@ gx_default_setup_buf_device(gx_device *bdev, byte *buffer, int bytes_per_line,
         mdev->foreign_line_pointers = false;
     }
     mdev->height = full_height;
-    code = gdev_mem_set_line_ptrs(mdev, buffer + raster * y, bytes_per_line,
+    code = gdev_mem_set_line_ptrs(mdev, buffer + raster * y, raster,
                                   ptrs, setup_height);
     mdev->height = setup_height;
     bdev->height = setup_height; /* do here in case mdev == bdev */
@@ -1318,14 +1547,12 @@ gdev_prn_close_printer(gx_device * pdev)
 
 /* compare two space_params, we can't do this with memcmp since there is padding in the structure */
 static int
-compare_gdev_prn_space_params(const gdev_prn_space_params sp1, 
+compare_gdev_prn_space_params(const gdev_prn_space_params sp1,
                               const gdev_prn_space_params sp2) {
   if (sp1.MaxBitmap != sp2.MaxBitmap)
     return(1);
   if (sp1.BufferSpace != sp2.BufferSpace)
     return(1);
-  if (sp1.band.page_uses_transparency != sp2.band.page_uses_transparency)
-    return(1);
   if (sp1.band.BandWidth != sp2.band.BandWidth)
     return(1);
   if (sp1.band.BandHeight != sp2.band.BandHeight)
@@ -1336,8 +1563,8 @@ compare_gdev_prn_space_params(const gdev_prn_space_params sp1,
     return(1);
   if (sp1.banding_type != sp2.banding_type)
     return(1);
- 
-  return(0); 
+
+  return(0);
 }
 
 
@@ -1370,10 +1597,10 @@ gdev_prn_maybe_realloc_memory(gx_device_printer *prdev,
         gdev_prn_space_params new_sp;
 
 #ifdef DEBUGGING_HACKS
-debug_dump_bytes((const byte *)old_sp, (const byte *)(old_sp + 1), "old");
-debug_dump_bytes((const byte *)&prdev->space_params,
+debug_dump_bytes(pdev->memory, (const byte *)old_sp, (const byte *)(old_sp + 1), "old");
+debug_dump_bytes(pddev->memory, (const byte *)&prdev->space_params,
                  (const byte *)(&prdev->space_params + 1), "new");
-dprintf4("w=%d/%d, h=%d/%d\n", old_width, new_width, old_height, new_height);
+dmprintf4(pdev->memory, "w=%d/%d, h=%d/%d\n", old_width, new_width, old_height, new_height);
 #endif /*DEBUGGING_HACKS*/
         new_sp = prdev->space_params;
         prdev->width = old_width;
