@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Client color structure definition */
 
 #ifndef gsccolor_INCLUDED
@@ -29,10 +31,30 @@ typedef struct gs_pattern_instance_s gs_pattern_instance_t;
  * Define the maximum number of components in a client color.
  * This must be at least 4, and should be at least 6 to accommodate
  * hexachrome DeviceN color spaces. We saw 9-component (bug 691002)
- * and 13-component (bug 691425) colors in the wild.
+ * and 13-component (bug 691425) colors in the wild.   Also define
+ * the maximum number of colorants that we will support within a
+ * single DeviceN color space.  AR supports 32.
  */
 #ifndef GS_CLIENT_COLOR_MAX_COMPONENTS		/* Allow override with XCFLAGS */
-#  define GS_CLIENT_COLOR_MAX_COMPONENTS (14)
+#  define GS_CLIENT_COLOR_MAX_COMPONENTS (64)
+#endif
+
+#ifndef MAX_COMPONENTS_IN_DEVN		
+#  define MAX_COMPONENTS_IN_DEVN (64)
+#endif
+
+/* There is a speed penalty for supporting lots of spot colors, so certain
+ * devices (tiffsep, tiffsep1, psdcmyk, etc) offer -dMaxSpots. This allows
+ * us to compile in a high 'hard' limit on the number of components
+ * (GS_CLIENT_COLOR_MAX_COMPONENTS) and yet to allow runtime selection of the
+ * real number to be used.
+ *
+ * GS_SOFT_MAX_SPOTS is the maximum number of spots we allow by default;
+ * currently this is set to 10, to match the (historical) limit of 14
+ * components that Ghostscript has shipped with for years.
+ */
+#ifndef GS_SOFT_MAX_SPOTS
+#define GS_SOFT_MAX_SPOTS 10
 #endif
 
 /* Paint (non-Pattern) colors */
@@ -79,4 +101,15 @@ typedef enum {
     OTHER
 } gs_transparency_color_t;
 
+
+/*
+ * Color model component polarity. An "unknown" value has been added to
+ * this enumeration.  This is used for devices and color spaces.
+ */
+typedef enum {
+    GX_CINFO_POLARITY_UNKNOWN = -1,
+    GX_CINFO_POLARITY_SUBTRACTIVE = 0,
+    GX_CINFO_POLARITY_ADDITIVE
+} gx_color_polarity_t;
+
 #endif /* gsccolor_INCLUDED */
