@@ -1,23 +1,24 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsexit.h,v 1.2 2010/07/10 22:02:28 Arabidopsis Exp $ */
+
 /* Declarations for exits */
 
 #ifndef gsexit_INCLUDED
 #  define gsexit_INCLUDED
 
-
 /** The client must provide this.
  *  normally they do exit cleanup and error messaging
  *  without calling system exit() returning to the caller.
@@ -29,7 +30,7 @@ int gs_to_exit(const gs_memory_t *mem, int exit_status);
  */
 int gs_to_exit_with_code(const gs_memory_t *mem, int exit_status, int code);
 
-/** The client must provide this.  
+/** The client must provide this.
  * After possible cleanup it may call gp_do_exit() which calls exit() in a platform
  * independent way.  This is a fatal error so returning is not a good idea.
  */
