@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id$ */
+
 /* setscreen operator for Ghostscript library */
 #include "memory_.h"
 #include "string_.h"
@@ -461,6 +463,7 @@ gx_sort_ht_order(gx_ht_bit * recs, uint N)
     for (i = 0; i < N; i++)
         recs[i].offset = i;
     qsort((void *)recs, N, sizeof(*recs), compare_samples);
+#ifndef GS_THREADSAFE
 #ifdef DEBUG
     if (gs_debug_c('H')) {
         uint i;
@@ -471,6 +474,7 @@ gx_sort_ht_order(gx_ht_bit * recs, uint N)
                       i, recs[i].offset, recs[i].mask);
     }
 #endif
+#endif
 }
 
 /*
@@ -560,11 +564,11 @@ gx_ht_construct_bits(gx_ht_order * porder)
         gx_ht_construct_bit(phb, porder->width, phb->offset);
 #ifdef DEBUG
     if (gs_debug_c('H')) {
-        dlprintf1("[H]Halftone order bits 0x%lx:\n", (ulong)porder->bit_data);
+        dmlprintf1(porder->data_memory, "[H]Halftone order bits 0x%lx:\n", (ulong)porder->bit_data);
         for (i = 0, phb = (gx_ht_bit *)porder->bit_data;
              i < porder->num_bits;
              i++, phb++)
-            dlprintf3("%4d: %u:0x%lx\n", i, phb->offset,
+            dmlprintf3(porder->data_memory, "%4d: %u:0x%lx\n", i, phb->offset,
                       (ulong) phb->mask);
     }
 #endif
@@ -1443,6 +1447,7 @@ gx_ht_construct_threshold( gx_ht_order *d_order, gx_device *dev,
     bool is_inverting = false;
     int num_repeat, shift;
     int row_kk, col_kk, kk;
+    int off;
 
     /* We can have simple or complete orders.  Simple ones tile the threshold
        with shifts.   To handle those we simply loop over the number of 
@@ -1501,6 +1506,9 @@ gx_ht_construct_threshold( gx_ht_order *d_order, gx_device *dev,
     hsize = d_order->num_levels;
     nshades = hsize * max_value + 1;
 
+    off = 128 / d_order->num_levels;
+    if (d_order->num_levels < 128)
+        init_value -= off-1;
     for( i = 0; i < d_order->num_bits; i++ ) {
         thresh[i] = init_value;
     }
@@ -1509,17 +1517,14 @@ gx_ht_construct_threshold( gx_ht_order *d_order, gx_device *dev,
     while (l < d_order->num_levels) {
         /* If we have some dots to turn on then proceed */
         if (d_order->levels[l] > d_order->levels[prev_l]) {
-            t_level = (256 * l) / d_order->num_levels;
+            t_level = (256 * l) / d_order->num_levels - off;
             t_level_adjust = byte2frac(t_level) * nshades / (frac_1_long + 1);
             delta = t_level_adjust - t_level;
+            t_level -= delta_sum;
             if (delta > delta_sum) {
                 /* We had a change in our value.  We need to do some adjustments.
-                   This particular level stays were it is and subsequent ones
-                   will be offset by delta_sum. */
-                t_level -= delta_sum;
+                   Subsequent levels ones will be offset by delta_sum. */
                 delta_sum += delta;
-            } else {
-                t_level -= delta_sum;
             }
             /* If needed, map through the inverse of the transfer function */
             if (have_transfer) {
@@ -1548,13 +1553,19 @@ gx_ht_construct_threshold( gx_ht_order *d_order, gx_device *dev,
         l++;
     }
     d_order->threshold = thresh;
+    if (dev->color_info.polarity == GX_CINFO_POLARITY_SUBTRACTIVE) {
+      for(i = 0; i < (int)d_order->height; i++ ) {
+         for( j=(int)d_order->width-1; j>=0; j-- )
+            *(thresh+j+(i*d_order->width)) = 255 - *(thresh+j+(i*d_order->width));
+      }
+    } 
 #ifdef DEBUG
    if ( gs_debug_c('h') ) {
       for( i=0; i<(int)d_order->height; i++ ) {
-         dprintf1("threshold array row %3d= ", i);
+         dmprintf1(memory, "threshold array row %3d= ", i);
          for( j=(int)d_order->width-1; j>=0; j-- )
-            dprintf1("%3d ", *(thresh+j+(i*d_order->width)) );
-         dprintf("\n");
+            dmprintf1(memory, "%3d ", *(thresh+j+(i*d_order->width)) );
+         dmprintf(memory, "\n");
       }
    }
 #endif
