@@ -24,6 +24,8 @@
 #include "lcms.h"
 #include "gslibctx.h"
 #include "gserrors.h"
+#include "gp.h"
+
 
 #define DUMP_CMS_BUFFER 0
 #define DEBUG_LCMS_MEM 0
@@ -71,13 +73,19 @@ gscms_get_numberclrtnames(gcmmhprofile_t profile)
 
 /* Get the nth colorant name in the clrt tag */
 char*
-gscms_get_clrtname(gcmmhprofile_t profile, int colorcount)
+gscms_get_clrtname(gcmmhprofile_t profile, int colorcount, gs_memory_t *memory)
 {
     LPcmsNAMEDCOLORLIST lcms_names;
+    int length;
+    char *name;
 
     lcms_names = cmsReadColorantTable(profile, icSigColorantTableTag);
     if (colorcount+1 > lcms_names->nColors) return(NULL);
-    return(lcms_names->List[colorcount].Name);
+    length = strlen(lcms_names->List[colorcount].Name);
+    name = (char*) gs_alloc_bytes(memory, length + 1, "gscms_get_clrtname");
+    if (name)
+        strcpy(name, lcms_names->List[colorcount].Name);
+    return name;
 }
 
 /* Get the device space associated with this profile */
@@ -106,7 +114,7 @@ gscms_get_profile_data_space(gcmmhprofile_t profile)
 
 /* Get ICC Profile handle from buffer */
 gcmmhprofile_t
-gscms_get_profile_handle_mem(unsigned char *buffer, unsigned int input_size)
+gscms_get_profile_handle_mem(gs_memory_t *mem, unsigned char *buffer, unsigned int input_size)
 {
     cmsSetErrorHandler((cmsErrorHandlerFunction) gscms_error);
     return(cmsOpenProfileFromMem(buffer,input_size));
@@ -114,7 +122,7 @@ gscms_get_profile_handle_mem(unsigned char *buffer, unsigned int input_size)
 
 /* Get ICC Profile handle from file ptr */
 gcmmhprofile_t
-gscms_get_profile_handle_file(const char *filename)
+gscms_get_profile_handle_file(gs_memory_t *mem, const char *filename)
 {
     return(cmsOpenProfileFromFile(filename, "r"));
 }
@@ -204,8 +212,8 @@ gscms_transform_color_buffer(gx_device *dev, gsicc_link_t *icclink,
         cmsDoTransform(hTransform,inputpos,outputpos,
                         input_buff_desc->plane_stride);
 #if DUMP_CMS_BUFFER
-        fid_in = fopen("CM_Input.raw","ab");
-        fid_out = fopen("CM_Output.raw","ab");
+        fid_in = gp_fopen("CM_Input.raw","ab");
+        fid_out = fp_fopen("CM_Output.raw","ab");
         fwrite((unsigned char*) inputbuffer,sizeof(unsigned char),
                                 input_buff_desc->plane_stride * 
                                 input_buff_desc->num_chan, fid_in);
@@ -224,8 +232,8 @@ gscms_transform_color_buffer(gx_device *dev, gsicc_link_t *icclink,
             outputpos += output_buff_desc->row_stride;
         }
 #if DUMP_CMS_BUFFER
-        fid_in = fopen("CM_Input.raw","ab");
-        fid_out = fopen("CM_Output.raw","ab");
+        fid_in = gp_fopen("CM_Input.raw","ab");
+        fid_out = gp_fopen("CM_Output.raw","ab");
         fwrite((unsigned char*) inputbuffer,sizeof(unsigned char),
                                 input_buff_desc->row_stride,fid_in);
         fwrite((unsigned char*) outputbuffer,sizeof(unsigned char),
@@ -264,8 +272,9 @@ gscms_transform_color(gx_device *dev, gsicc_link_t *icclink, void *inputcolor,
 /* Get the link from the CMS. TODO:  Add error checking */
 gcmmhlink_t
 gscms_get_link(gcmmhprofile_t  lcms_srchandle,
-                    gcmmhprofile_t lcms_deshandle,
-                    gsicc_rendering_param_t *rendering_params)
+               gcmmhprofile_t lcms_deshandle,
+               gsicc_rendering_param_t *rendering_params, int cmm_flags,
+               gs_memory_t *mem)
 {
     DWORD src_data_type,des_data_type;
     icColorSpaceSignature src_color_space,des_color_space;
@@ -297,7 +306,7 @@ gscms_get_link(gcmmhprofile_t  lcms_srchandle,
 /* Create the link */
     return(cmsCreateTransform(lcms_srchandle, src_data_type, lcms_deshandle,
                         des_data_type, rendering_params->rendering_intent,
-               (cmsFLAGS_BLACKPOINTCOMPENSATION | cmsFLAGS_HIGHRESPRECALC |
+               (cmm_flags | cmsFLAGS_BLACKPOINTCOMPENSATION | cmsFLAGS_HIGHRESPRECALC |
                 cmsFLAGS_NOTCACHE )));
     /* cmsFLAGS_HIGHRESPRECALC)  cmsFLAGS_NOTPRECALC  cmsFLAGS_LOWRESPRECALC*/
 }
@@ -309,7 +318,9 @@ gscms_get_link_proof_devlink(gcmmhprofile_t lcms_srchandle,
                              gcmmhprofile_t lcms_proofhandle,
                              gcmmhprofile_t lcms_deshandle, 
                              gcmmhprofile_t lcms_devlinkhandle, 
-                             gsicc_rendering_param_t *rendering_params)
+                             gsicc_rendering_param_t *rendering_params,
+                             bool src_dev_link, int cmm_flags,
+                             gs_memory_t *mem)
 {
     DWORD src_data_type,des_data_type;
     icColorSpaceSignature src_color_space,des_color_space;
@@ -353,22 +364,23 @@ gscms_get_link_proof_devlink(gcmmhprofile_t lcms_srchandle,
     }
     return(cmsCreateMultiprofileTransform(hProfiles, nProfiles, src_data_type, 
                                           des_data_type, rendering_params->rendering_intent, 
-                                          (cmsFLAGS_BLACKPOINTCOMPENSATION | 
+                                          (cmm_flags | cmsFLAGS_BLACKPOINTCOMPENSATION | 
                                            cmsFLAGS_HIGHRESPRECALC |
                                            cmsFLAGS_NOTCACHE)));
 }
 
 /* Do any initialization if needed to the CMS */
-void
-gscms_create(void **contextptr)
+int
+gscms_create(gs_memory_t *memory)
 {
     /* Set our own error handling function */
     cmsSetErrorHandler((cmsErrorHandlerFunction) gscms_error);
+    return 0;
 }
 
 /* Do any clean up when done with the CMS if needed */
 void
-gscms_destroy(void **contextptr)
+gscms_destroy(gs_memory_t *memory)
 {
     /* Nothing to do here for lcms */
 }
@@ -485,7 +497,11 @@ void* _cmsMalloc(unsigned int size)
 {
     void *ptr;
 
+#if defined(SHARE_LCMS) && SHARE_LCMS==1
+    ptr = malloc(size);
+#else
     ptr = gs_alloc_bytes(gs_lib_ctx_get_non_gc_memory_t(), size, "lcms");
+#endif
     gs_warn2("lcms malloc (%d) at 0x%x",size,ptr);
     return ptr;
 }
@@ -494,7 +510,14 @@ void* _cmsCalloc(unsigned int nelts, unsigned int size)
 {
     void *ptr;
 
+#if defined(SHARE_LCMS) && SHARE_LCMS==1
+    ptr = malloc(nelts * size);
+#else
     ptr = gs_alloc_byte_array(gs_lib_ctx_get_non_gc_memory_t(), nelts, size, "lcms");
+#endif
+
+    if (ptr != NULL)
+        memset(ptr, 0, nelts * size);
     gs_warn2("lcms calloc (%d) at 0x%x",nelts*size,ptr);
     return ptr;
 }
@@ -503,7 +526,11 @@ void _cmsFree(void *ptr)
 {
     if (ptr != NULL) {
         gs_warn1("lcms free at 0x%x",ptr);
+#if defined(SHARE_LCMS) && SHARE_LCMS==1
+        free(ptr);
+#else
         gs_free_object(gs_lib_ctx_get_non_gc_memory_t(), ptr, "lcms");
+#endif
     }
 }
 
@@ -511,17 +538,35 @@ void _cmsFree(void *ptr)
 
 void* _cmsMalloc(unsigned int size)
 {
+#if defined(SHARE_LCMS) && SHARE_LCMS==1
+    return malloc(size);
+#else
     return gs_alloc_bytes(gs_lib_ctx_get_non_gc_memory_t(), size, "lcms");
+#endif
 }
 
 void* _cmsCalloc(unsigned int nelts, unsigned int size)
 {
-    return gs_alloc_byte_array(gs_lib_ctx_get_non_gc_memory_t(), nelts, size, "lcms");
+    void *ptr;
+
+#if defined(SHARE_LCMS) && SHARE_LCMS==1
+    ptr = malloc(nelts * size);
+#else
+    ptr = gs_alloc_byte_array(gs_lib_ctx_get_non_gc_memory_t(), nelts, size, "lcms");
+#endif
+
+    if (ptr != NULL)
+        memset(ptr, 0, nelts * size);
+    return ptr;
 }
 
 void _cmsFree(void *ptr)
 {
+#if defined(SHARE_LCMS) && SHARE_LCMS==1
+    free(ptr);
+#else
     if (ptr != NULL)
         gs_free_object(gs_lib_ctx_get_non_gc_memory_t(), ptr, "lcms");
+#endif
 }
 #endif
