@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Implementation of color spaces */
 /* Requires gsstruct.h */
 
@@ -209,6 +211,15 @@ struct gs_color_space_type_s {
 #define cs_is_linear(pcs, pis, dev, c0, c1, c2, c3, smoothness, icclink)\
   (*(pcs)->type->is_linear)(pcs, pis, dev, c0, c1, c2, c3, smoothness, icclink)
         cs_proc_is_linear((*is_linear));
+
+    /* Define the polarity of a color space.  We will use gx_color_polarity_t
+       with spaces such as patterns returning GX_CINFO_POLARITY_UNKNOWN */
+
+#define cs_proc_polarity(proc)\
+  gx_color_polarity_t proc(const gs_color_space *)
+#define cs_polarity(pcs)\
+  (*(pcs)->type->polarity)(pcs)
+        cs_proc_polarity((*polarity));
 };
 
 extern_st(st_base_color_space);
@@ -220,6 +231,9 @@ extern_st(st_base_color_space);
 cs_proc_num_components(gx_num_components_1);
 cs_proc_num_components(gx_num_components_3);
 cs_proc_num_components(gx_num_components_4);
+cs_proc_polarity(gx_polarity_subtractive);
+cs_proc_polarity(gx_polarity_additive);
+cs_proc_polarity(gx_polarity_unknown);
 cs_proc_init_color(gx_init_paint_1);
 cs_proc_init_color(gx_init_paint_3);
 cs_proc_init_color(gx_init_paint_4);
@@ -232,6 +246,7 @@ cs_proc_concretize_color(gx_no_concretize_color);
 cs_proc_remap_color(gx_default_remap_color);
 cs_proc_install_cspace(gx_no_install_cspace);
 cs_proc_set_overprint(gx_spot_colors_set_overprint);
+cs_proc_set_overprint(gx_simulated_set_overprint);
 cs_proc_adjust_color_count(gx_no_adjust_color_count);
 cs_proc_serialize(gx_serialize_cspace_type);
 cs_proc_is_linear(gx_cspace_no_linear);
@@ -259,10 +274,13 @@ gs_color_space *
 gs_cspace_alloc(gs_memory_t *mem, const gs_color_space_type *pcstype);
 
 /* Determine if the current color model is a "DeviceCMYK" color model, and */
-/* if so what are its process color components. */
+/* if so what are its process color components. Also the same for the RGB */
+/* device that simulates CMYK overprinting */
 gx_color_index check_cmyk_color_model_comps(gx_device * dev);
+gx_color_index check_rgb_color_model_comps(gx_device * dev);
 
 /* Shared code with ICC overprint */
 int gx_set_overprint_cmyk(const gs_color_space * pcs, gs_state * pgs);
+int gx_set_overprint_rgb(const gs_color_space * pcs, gs_state * pgs);
 
 #endif /* gxcspace_INCLUDED */
