@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* PatternType 1 filling algorithms */
 #include "string_.h"
 #include "math_.h"
@@ -30,6 +32,7 @@
 #include "gxcldev.h"
 #include "gxblend.h"
 #include "gsicc_cache.h"
+#include "gxdevsop.h"
 
 #include "gdevp14.h"
 
@@ -47,10 +50,12 @@ typedef struct tile_fill_state_s {
 
     /* Variables set at initialization */
 
-    gx_device_tile_clip cdev;
-    gx_device *pcdev;           /* original device or &cdev */
+    gx_device_tile_clip *cdev;
+    gx_device *pcdev;           /* original device or cdev */
     const gx_strip_bitmap *tmask;
     gs_int_point phase;
+    int num_planes;    /* negative if not planar */
+
 
     /* Following are only for uncolored patterns */
 
@@ -58,7 +63,6 @@ typedef struct tile_fill_state_s {
 
     /* Following are only for colored patterns */
 
-    const gx_rop_source_t *rop_source;
     gx_device *orig_dev;
     int xoff, yoff;             /* set dynamically */
 
@@ -83,6 +87,11 @@ typedef struct tile_fill_trans_state_s {
 
 } tile_fill_trans_state_t;
 
+/* we need some means of detecting if a forwarding clipping device was
+   installed.  If the tile state cdev pointer is non-NULL, then the
+   target output device must be the clipping device. */
+#define CLIPDEV_INSTALLED (state.cdev != NULL)
+
 /* Initialize the filling state. */
 static int
 tile_fill_init(tile_fill_state_t * ptfs, const gx_device_color * pdevc,
@@ -90,14 +99,29 @@ tile_fill_init(tile_fill_state_t * ptfs, const gx_device_color * pdevc,
 {
     gx_color_tile *m_tile = pdevc->mask.m_tile;
     int px, py;
+    bool is_planar;
 
     ptfs->pdevc = pdevc;
+    is_planar = dev->is_planar;
+    if (is_planar) {
+        ptfs->num_planes = dev->color_info.num_components;
+    } else {
+        ptfs->num_planes = -1;
+    }
     if (m_tile == 0) {          /* no clipping */
+        ptfs->cdev = NULL;
         ptfs->pcdev = dev;
         ptfs->phase = pdevc->phase;
         return 0;
     }
-    ptfs->pcdev = (gx_device *) & ptfs->cdev;
+    if ((ptfs->cdev = (gx_device_tile_clip *)gs_alloc_struct(dev->memory,
+                                                 gx_device_tile_clip,
+                                                 &st_device_tile_clip,
+                                                 "tile_fill_init(cdev)")) == NULL) {
+        return_error(gs_error_VMerror);
+    }
+    ptfs->cdev->finalize = NULL;
+    ptfs->pcdev = (gx_device *)ptfs->cdev;
     ptfs->tmask = &m_tile->tmask;
     ptfs->phase.x = pdevc->mask.m_phase.x;
     ptfs->phase.y = pdevc->mask.m_phase.y;
@@ -112,7 +136,7 @@ tile_fill_init(tile_fill_state_t * ptfs, const gx_device_color * pdevc,
                   m_tile->tmask.rep_height);
     } else
         px = py = 0;
-    return tile_clip_initialize(&ptfs->cdev, ptfs->tmask, dev, px, py, NULL);
+    return tile_clip_initialize(ptfs->cdev, ptfs->tmask, dev, px, py);
 }
 
 /*
@@ -133,6 +157,9 @@ tile_by_steps(tile_fill_state_t * ptfs, int x0, int y0, int w0, int h0,
     int i0, i1, j0, j1, i, j;
     gs_matrix step_matrix;      /* translated by phase */
     int code;
+#ifdef DEBUG
+    const gs_memory_t *mem = ptfs->pcdev->memory;
+#endif
 
     ptfs->x0 = x0, ptfs->w0 = w0;
     ptfs->y0 = y0, ptfs->h0 = h0;
@@ -149,11 +176,11 @@ tile_by_steps(tile_fill_state_t * ptfs, int x0, int y0, int w0, int h0,
         bbox.p.x = x0, bbox.p.y = y0;
         bbox.q.x = x1, bbox.q.y = y1;
         gs_bbox_transform_inverse(&bbox, &step_matrix, &ibbox);
-        if_debug10('T',
+        if_debug10m('T', mem,
           "[T]x,y=(%d,%d) w,h=(%d,%d) => (%g,%g),(%g,%g), offset=(%g,%g)\n",
-                   x0, y0, w0, h0,
-                   ibbox.p.x, ibbox.p.y, ibbox.q.x, ibbox.q.y,
-                   step_matrix.tx, step_matrix.ty);
+                    x0, y0, w0, h0,
+                    ibbox.p.x, ibbox.p.y, ibbox.q.x, ibbox.q.y,
+                    step_matrix.tx, step_matrix.ty);
         /*
          * If the pattern is partly transparent and XStep/YStep is smaller
          * than the device space BBox, we need to ensure that we cover
@@ -171,7 +198,7 @@ tile_by_steps(tile_fill_state_t * ptfs, int x0, int y0, int w0, int h0,
         i1 = (int)ceil(u1);
         j1 = (int)ceil(v1);
     }
-    if_debug4('T', "[T]i=(%d,%d) j=(%d,%d)\n", i0, i1, j0, j1);
+    if_debug4m('T', mem, "[T]i=(%d,%d) j=(%d,%d)\n", i0, i1, j0, j1);
     for (i = i0; i < i1; i++)
         for (j = j0; j < j1; j++) {
             int x = (int)fastfloor(step_matrix.xx * i +
@@ -182,7 +209,7 @@ tile_by_steps(tile_fill_state_t * ptfs, int x0, int y0, int w0, int h0,
             int h = tbits_or_tmask->size.y;
             int xoff, yoff;
 
-            if_debug4('T', "[T]i=%d j=%d x,y=(%d,%d)", i, j, x, y);
+            if_debug4m('T', mem, "[T]i=%d j=%d x,y=(%d,%d)", i, j, x, y);
             if (x < x0)
                 xoff = x0 - x, x = x0, w -= xoff;
             else
@@ -195,11 +222,11 @@ tile_by_steps(tile_fill_state_t * ptfs, int x0, int y0, int w0, int h0,
                 w = x1 - x;
             if (y + h > y1)
                 h = y1 - y;
-            if_debug6('T', "=>(%d,%d) w,h=(%d,%d) x/yoff=(%d,%d)\n",
-                      x, y, w, h, xoff, yoff);
+            if_debug6m('T', mem, "=>(%d,%d) w,h=(%d,%d) x/yoff=(%d,%d)\n",
+                       x, y, w, h, xoff, yoff);
             if (w > 0 && h > 0) {
-                if (ptfs->pcdev == (gx_device *) & ptfs->cdev)
-                    tile_clip_set_phase(&ptfs->cdev,
+                if (ptfs->pcdev == (gx_device *)ptfs->cdev)
+                    tile_clip_set_phase(ptfs->cdev,
                                 imod(xoff - x, ptfs->tmask->rep_width),
                                 imod(yoff - y, ptfs->tmask->rep_height));
                 /* Set the offsets for colored pattern fills */
@@ -222,42 +249,67 @@ tile_colored_fill(const tile_fill_state_t * ptfs,
     gx_color_tile *ptile = ptfs->pdevc->colors.pattern.p_tile;
     gs_logical_operation_t lop = ptfs->lop;
     const gx_rop_source_t *source = ptfs->source;
-    const gx_rop_source_t *rop_source = ptfs->rop_source;
     gx_device *dev = ptfs->orig_dev;
     int xoff = ptfs->xoff, yoff = ptfs->yoff;
     gx_strip_bitmap *bits = &ptile->tbits;
     const byte *data = bits->data;
     bool full_transfer = (w == ptfs->w0 && h == ptfs->h0);
-    gx_bitmap_id source_id =
-    (full_transfer ? rop_source->id : gx_no_bitmap_id);
-    int code;
+    int code = 0;
 
-    if (source == NULL && lop_no_S_is_T(lop))
-        code = (*dev_proc(ptfs->pcdev, copy_color))
-            (ptfs->pcdev, data + bits->raster * yoff, xoff,
-             bits->raster,
-             (full_transfer ? bits->id : gx_no_bitmap_id),
-             x, y, w, h);
-    else {
+    if (source == NULL && lop_no_S_is_T(lop) && dev->procs.copy_planes != NULL &&
+        ptfs->num_planes > 0) {
+            code = (*dev_proc(ptfs->pcdev, copy_planes))
+                    (ptfs->pcdev, data + bits->raster * yoff, xoff,
+                     bits->raster,
+                     (full_transfer ? bits->id : gx_no_bitmap_id),
+                     x, y, w, h, ptile->tbits.rep_height);
+    } else if (source == NULL && lop_no_S_is_T(lop)) {
+            code = (*dev_proc(ptfs->pcdev, copy_color))
+                    (ptfs->pcdev, data + bits->raster * yoff, xoff,
+                     bits->raster,
+                     (full_transfer ? bits->id : gx_no_bitmap_id),
+                     x, y, w, h);
+    } else {
         gx_strip_bitmap data_tile;
+        gx_bitmap_id source_id;
+        gx_rop_source_t no_source;
 
+        if (source == NULL)
+            set_rop_no_source(source, no_source, dev);
+        source_id = (full_transfer ? source->id : gx_no_bitmap_id);
         data_tile.data = (byte *) data;         /* actually const */
         data_tile.raster = bits->raster;
         data_tile.size.x = data_tile.rep_width = ptile->tbits.size.x;
         data_tile.size.y = data_tile.rep_height = ptile->tbits.size.y;
         data_tile.id = bits->id;
         data_tile.shift = data_tile.rep_shift = 0;
-        code = (*dev_proc(dev, strip_copy_rop))
-            (dev,
-             rop_source->sdata + (y - ptfs->y0) * rop_source->sraster,
-             rop_source->sourcex + (x - ptfs->x0),
-             rop_source->sraster, source_id,
-             (rop_source->use_scolors ? rop_source->scolors : NULL),
-             &data_tile, NULL,
-             x, y, w, h,
-             imod(xoff - x, data_tile.rep_width),
-             imod(yoff - y, data_tile.rep_height),
-             lop);
+        data_tile.num_planes = (ptfs->num_planes > 1 ? ptfs->num_planes : 1);
+        if (source->planar_height == 0) {
+            code = (*dev_proc(ptfs->pcdev, strip_copy_rop))
+                           (ptfs->pcdev,
+                            source->sdata + (y - ptfs->y0) * source->sraster,
+                            source->sourcex + (x - ptfs->x0),
+                            source->sraster, source_id,
+                            (source->use_scolors ? source->scolors : NULL),
+                            &data_tile, NULL,
+                            x, y, w, h,
+                            imod(xoff - x, data_tile.rep_width),
+                            imod(yoff - y, data_tile.rep_height),
+                            lop);
+        } else {
+            code = (*dev_proc(ptfs->pcdev, strip_copy_rop2))
+                           (ptfs->pcdev,
+                            source->sdata + (y - ptfs->y0) * source->sraster,
+                            source->sourcex + (x - ptfs->x0),
+                            source->sraster, source_id,
+                            (source->use_scolors ? source->scolors : NULL),
+                            &data_tile, NULL,
+                            x, y, w, h,
+                            imod(xoff - x, data_tile.rep_width),
+                            imod(yoff - y, data_tile.rep_height),
+                            lop,
+                            source->planar_height);
+        }
     }
     return code;
 }
@@ -282,11 +334,11 @@ tile_pattern_clist(const tile_fill_state_t * ptfs,
         code = clist_read_icctable(crdev);
     /* Also allocate the icc cache for the clist reader */
     if ( crdev->icc_cache_cl == NULL )
-        crdev->icc_cache_cl = gsicc_cache_new(crdev->memory);
-    if_debug0('L', "Pattern clist playback begin\n");
+        crdev->icc_cache_cl = gsicc_cache_new(crdev->memory->thread_safe_memory);
+    if_debug0m('L', dev->memory, "Pattern clist playback begin\n");
     code = clist_playback_file_bands(playback_action_render,
                 crdev, &crdev->page_info, dev, 0, 0, ptfs->xoff - x, ptfs->yoff - y);
-    if_debug0('L', "Pattern clist playback end\n");
+    if_debug0m('L', dev->memory, "Pattern clist playback end\n");
     return code;
 }
 
@@ -309,9 +361,7 @@ gx_dc_pattern_fill_rectangle(const gx_device_color * pdevc, int x, int y,
         set_rop_no_source(rop_source, no_source, dev);
     bits = &ptile->tbits;
 
-    state.cdev.finalize = 0;
-
-    code = tile_fill_init(&state, pdevc, dev, false);
+    code = tile_fill_init(&state, pdevc, dev, false);	/* This _may_ allocate state.cdev */
     if (code < 0)
         return code;
     if (ptile->is_simple && ptile->cdev == NULL) {
@@ -322,23 +372,30 @@ gx_dc_pattern_fill_rectangle(const gx_device_color * pdevc, int x, int y,
             imod(-(int)fastfloor(ptile->step_matrix.ty - state.phase.y + 0.5),
                  bits->rep_height);
 
-        if (state.pcdev != dev)
-            tile_clip_set_phase(&state.cdev, px, py);
+        if (CLIPDEV_INSTALLED)
+            tile_clip_set_phase(state.cdev, px, py);
         if (source == NULL && lop_no_S_is_T(lop))
             code = (*dev_proc(state.pcdev, strip_tile_rectangle))
                 (state.pcdev, bits, x, y, w, h,
                  gx_no_color_index, gx_no_color_index, px, py);
-        else
+        else if (rop_source->planar_height == 0)
             code = (*dev_proc(state.pcdev, strip_copy_rop))
-                (state.pcdev,
-                 rop_source->sdata, rop_source->sourcex,
-                 rop_source->sraster, rop_source->id,
-                 (rop_source->use_scolors ? rop_source->scolors : NULL),
-                 bits, NULL, x, y, w, h, px, py, lop);
+                        (state.pcdev,
+                         rop_source->sdata, rop_source->sourcex,
+                         rop_source->sraster, rop_source->id,
+                         (rop_source->use_scolors ? rop_source->scolors : NULL),
+                         bits, NULL, x, y, w, h, px, py, lop);
+        else
+            code = (*dev_proc(state.pcdev, strip_copy_rop2))
+                        (state.pcdev,
+                         rop_source->sdata, rop_source->sourcex,
+                         rop_source->sraster, rop_source->id,
+                         (rop_source->use_scolors ? rop_source->scolors : NULL),
+                         bits, NULL, x, y, w, h, px, py, lop,
+                         rop_source->planar_height);
     } else {
         state.lop = lop;
         state.source = source;
-        state.rop_source = rop_source;
         state.orig_dev = dev;
         if (ptile->cdev == NULL) {
             code = tile_by_steps(&state, x, y, w, h, ptile,
@@ -361,8 +418,10 @@ gx_dc_pattern_fill_rectangle(const gx_device_color * pdevc, int x, int y,
                                  &tbits, tile_pattern_clist);
         }
     }
-    if(state.cdev.finalize)
-        state.cdev.finalize((gx_device *)&state.cdev);
+    if (CLIPDEV_INSTALLED) {
+        tile_clip_free((gx_device_tile_clip *)state.cdev);
+        state.cdev = NULL;
+    }
     return code;
 }
 
@@ -386,6 +445,7 @@ tile_masked_fill(const tile_fill_state_t * ptfs,
                           source->id : gx_no_bitmap_id);
         step_source.scolors[0] = source->scolors[0];
         step_source.scolors[1] = source->scolors[1];
+        step_source.planar_height = source->planar_height;
         step_source.use_scolors = source->use_scolors;
         return (*ptfs->fill_rectangle)
             (ptfs->pdevc, x, y, w, h, ptfs->pcdev, ptfs->lop, &step_source);
@@ -409,16 +469,67 @@ gx_dc_pure_masked_fill_rect(const gx_device_color * pdevc,
     if (code < 0)
         return code;
     if (state.pcdev == dev || ptile->is_simple)
-        return (*gx_dc_type_data_pure.fill_rectangle)
+        code = (*gx_dc_type_data_pure.fill_rectangle)
             (pdevc, x, y, w, h, state.pcdev, lop, source);
     else {
         state.lop = lop;
         state.source = source;
         state.fill_rectangle = gx_dc_type_data_pure.fill_rectangle;
-        return tile_by_steps(&state, x, y, w, h, ptile, &ptile->tmask,
+        code = tile_by_steps(&state, x, y, w, h, ptile, &ptile->tmask,
                              tile_masked_fill);
     }
+    if (CLIPDEV_INSTALLED) {
+        tile_clip_free((gx_device_tile_clip *)state.cdev);
+        state.cdev = NULL;
+    }
+    return code;
 }
+
+int
+gx_dc_devn_masked_fill_rect(const gx_device_color * pdevc,
+                            int x, int y, int w, int h, gx_device * dev,
+                            gs_logical_operation_t lop,
+                            const gx_rop_source_t * source)
+{
+    gx_color_tile *ptile = pdevc->mask.m_tile;
+    tile_fill_state_t state;
+    int code;
+
+    /*
+     * This routine should never be called if there is no masking,
+     * but we leave the checks below just in case.
+     */
+    code = tile_fill_init(&state, pdevc, dev, true);
+    if (code < 0)
+        return code;
+    if (state.pcdev == dev || ptile->is_simple) {
+        gx_device_color dcolor = *pdevc;
+
+        if (ptile == NULL) {
+        int k;
+
+            /* Have to set the pdevc to a non mask type since the pattern was stored as non-masking */
+            dcolor.type = gx_dc_type_devn;
+            for (k = 0; k < GS_CLIENT_COLOR_MAX_COMPONENTS; k++) {
+                dcolor.colors.devn.values[k] = pdevc->colors.devn.values[k];
+            }
+        }
+        code = (*gx_dc_type_data_devn.fill_rectangle)
+            (&dcolor, x, y, w, h, state.pcdev, lop, source);
+    } else {
+        state.lop = lop;
+        state.source = source;
+        state.fill_rectangle = gx_dc_type_data_devn.fill_rectangle;
+        code = tile_by_steps(&state, x, y, w, h, ptile, &ptile->tmask,
+                             tile_masked_fill);
+    }
+    if (CLIPDEV_INSTALLED) {
+        tile_clip_free((gx_device_tile_clip *)state.cdev);
+        state.cdev = NULL;
+    }
+    return code;
+}
+
 int
 gx_dc_binary_masked_fill_rect(const gx_device_color * pdevc,
                               int x, int y, int w, int h, gx_device * dev,
@@ -433,16 +544,22 @@ gx_dc_binary_masked_fill_rect(const gx_device_color * pdevc,
     if (code < 0)
         return code;
     if (state.pcdev == dev || ptile->is_simple)
-        return (*gx_dc_type_data_ht_binary.fill_rectangle)
+        code = (*gx_dc_type_data_ht_binary.fill_rectangle)
             (pdevc, x, y, w, h, state.pcdev, lop, source);
     else {
         state.lop = lop;
         state.source = source;
         state.fill_rectangle = gx_dc_type_data_ht_binary.fill_rectangle;
-        return tile_by_steps(&state, x, y, w, h, ptile, &ptile->tmask,
+        code = tile_by_steps(&state, x, y, w, h, ptile, &ptile->tmask,
                              tile_masked_fill);
     }
+    if (CLIPDEV_INSTALLED) {
+        tile_clip_free((gx_device_tile_clip *)state.cdev);
+        state.cdev = NULL;
+    }
+    return code;
 }
+
 int
 gx_dc_colored_masked_fill_rect(const gx_device_color * pdevc,
                                int x, int y, int w, int h, gx_device * dev,
@@ -457,15 +574,20 @@ gx_dc_colored_masked_fill_rect(const gx_device_color * pdevc,
     if (code < 0)
         return code;
     if (state.pcdev == dev || ptile->is_simple)
-        return (*gx_dc_type_data_ht_colored.fill_rectangle)
+        code = (*gx_dc_type_data_ht_colored.fill_rectangle)
             (pdevc, x, y, w, h, state.pcdev, lop, source);
     else {
         state.lop = lop;
         state.source = source;
         state.fill_rectangle = gx_dc_type_data_ht_colored.fill_rectangle;
-        return tile_by_steps(&state, x, y, w, h, ptile, &ptile->tmask,
+        code = tile_by_steps(&state, x, y, w, h, ptile, &ptile->tmask,
                              tile_masked_fill);
     }
+    if (CLIPDEV_INSTALLED) {
+        tile_clip_free((gx_device_tile_clip *)state.cdev);
+        state.cdev = NULL;
+    }
+    return code;
 }
 
 /*
@@ -481,6 +603,9 @@ tile_by_steps_trans(tile_fill_trans_state_t * ptfs, int x0, int y0, int w0, int
     int i0, i1, j0, j1, i, j;
     gs_matrix step_matrix;      /* translated by phase */
     gx_pattern_trans_t *ptrans_pat = ptile->ttrans;
+#ifdef DEBUG
+    const gs_memory_t *mem = ptile->ttrans->mem;
+#endif
 
     ptfs->x0 = x0, ptfs->w0 = w0;
     ptfs->y0 = y0, ptfs->h0 = h0;
@@ -497,11 +622,11 @@ tile_by_steps_trans(tile_fill_trans_state_t * ptfs, int x0, int y0, int w0, int
         bbox.p.x = x0, bbox.p.y = y0;
         bbox.q.x = x1, bbox.q.y = y1;
         gs_bbox_transform_inverse(&bbox, &step_matrix, &ibbox);
-        if_debug10('T',
+        if_debug10m('T', mem,
           "[T]x,y=(%d,%d) w,h=(%d,%d) => (%g,%g),(%g,%g), offset=(%g,%g)\n",
-                   x0, y0, w0, h0,
-                   ibbox.p.x, ibbox.p.y, ibbox.q.x, ibbox.q.y,
-                   step_matrix.tx, step_matrix.ty);
+                    x0, y0, w0, h0,
+                    ibbox.p.x, ibbox.p.y, ibbox.q.x, ibbox.q.y,
+                    step_matrix.tx, step_matrix.ty);
         /*
          * If the pattern is partly transparent and XStep/YStep is smaller
          * than the device space BBox, we need to ensure that we cover
@@ -519,7 +644,7 @@ tile_by_steps_trans(tile_fill_trans_state_t * ptfs, int x0, int y0, int w0, int
         i1 = (int)ceil(u1);
         j1 = (int)ceil(v1);
     }
-    if_debug4('T', "[T]i=(%d,%d) j=(%d,%d)\n", i0, i1, j0, j1);
+    if_debug4m('T', mem, "[T]i=(%d,%d) j=(%d,%d)\n", i0, i1, j0, j1);
     for (i = i0; i < i1; i++)
         for (j = j0; j < j1; j++) {
             int x = (int)fastfloor(step_matrix.xx * i +
@@ -531,7 +656,7 @@ tile_by_steps_trans(tile_fill_trans_state_t * ptfs, int x0, int y0, int w0, int
             int xoff, yoff;
             int px, py;
 
-            if_debug4('T', "[T]i=%d j=%d x,y=(%d,%d)", i, j, x, y);
+            if_debug4m('T', mem, "[T]i=%d j=%d x,y=(%d,%d)", i, j, x, y);
             if (x < x0)
                 xoff = x0 - x, x = x0, w -= xoff;
             else
@@ -544,8 +669,8 @@ tile_by_steps_trans(tile_fill_trans_state_t * ptfs, int x0, int y0, int w0, int
                 w = x1 - x;
             if (y + h > y1)
                 h = y1 - y;
-            if_debug6('T', "=>(%d,%d) w,h=(%d,%d) x/yoff=(%d,%d)\n",
-                      x, y, w, h, xoff, yoff);
+            if_debug6m('T', mem, "=>(%d,%d) w,h=(%d,%d) x/yoff=(%d,%d)\n",
+                       x, y, w, h, xoff, yoff);
             if (w > 0 && h > 0) {
 
                 px = imod(xoff - x, ptile->ttrans->width);
@@ -821,16 +946,24 @@ gx_dc_pat_trans_fill_rectangle(const gx_device_color * pdevc, int x, int y,
     phase.x = pdevc->phase.x;
     phase.y = pdevc->phase.y;
 
+#if 0
+    if_debug8m('v', ptile->ttrans->mem,
+            "[v]gx_dc_pat_trans_fill_rectangle, Fill: (%d, %d), %d x %d To Buffer: (%d, %d), %d x %d \n",
+            x, y, w, h,  ptile->ttrans->fill_trans_buffer->rect.p.x, 
+            ptile->ttrans->fill_trans_buffer->rect.p.y, 
+            ptile->ttrans->fill_trans_buffer->rect.q.x - 
+            ptile->ttrans->fill_trans_buffer->rect.p.x,
+            ptile->ttrans->fill_trans_buffer->rect.q.y - 
+            ptile->ttrans->fill_trans_buffer->rect.p.y);
+#endif
     code = gx_trans_pattern_fill_rect(x, y, x+w, y+h, ptile,
                                       ptile->ttrans->fill_trans_buffer, phase,
                                       dev, pdevc);
-
     return code;
 }
 
-/* This fills the transparency buffer rectangles with a pattern
-   buffer that includes transparency */
-
+/* This fills the transparency buffer rectangles with a pattern buffer
+   that includes transparency */
 int
 gx_trans_pattern_fill_rect(int xmin, int ymin, int xmax, int ymax,
                            gx_color_tile *ptile,
@@ -838,21 +971,21 @@ gx_trans_pattern_fill_rect(int xmin, int ymin, int xmax, int ymax,
                            gs_int_point phase, gx_device *dev,
                            const gx_device_color * pdevc)
 {
-
     tile_fill_trans_state_t state_trans;
     tile_fill_state_t state_clist_trans;
-    int code;
+    int code = 0;
+    int w = xmax - xmin;
+    int h = ymax - ymin;
+    int xmax_new, ymax_new;
 
     if (ptile == 0)             /* null pattern */
         return 0;
-
-    /* Fit fill */
-    if ( (xmin | ymin) < 0 ) {
-        if ( xmin < 0 )
-            xmin = 0;
-        if ( ymin < 0 )
-            ymin = 0;
-    }
+   
+    fit_fill_xywh(dev, xmin, ymin, w, h);
+    if (w < 0 || h < 0) 
+        return 0;
+    xmax = w + xmin;
+    ymax = h + ymin;
 
     /* Initialize the fill state */
     state_trans.phase.x = phase.x;
@@ -877,6 +1010,7 @@ gx_trans_pattern_fill_rect(int xmin, int ymin, int xmax, int ymax,
                transformed bbox */
             code = tile_by_steps_trans(&state_trans, xmin, ymin, xmax-xmin,
                                         ymax-ymin, fill_trans_buffer, ptile);
+
         } else {
             /* clist for the trans tile.  This uses the pdf14 device as a target
                and should blend directly into the buffer.  Note that the
@@ -902,9 +1036,16 @@ gx_trans_pattern_fill_rect(int xmin, int ymin, int xmax, int ymax,
             tbits = ptile->tbits;
             tbits.size.x = crdev->width;
             tbits.size.y = crdev->height;
-            code = tile_by_steps(&state_clist_trans, xmin, ymin, xmax,
-                                 ymax, ptile, &tbits, tile_pattern_clist);
+            if (code >= 0)
+                code = tile_by_steps(&state_clist_trans, xmin, ymin, xmax,
+                                     ymax, ptile, &tbits, tile_pattern_clist);
+
+            if (code >= 0 && (state_clist_trans.cdev != NULL)) {
+                tile_clip_free((gx_device_tile_clip *)state_clist_trans.cdev);
+                state_clist_trans.cdev = NULL;
+            }
+
         }
     }
-    return(0);
+    return code;
 }
