@@ -1,21 +1,24 @@
-/* Copyright (C) 2011 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
+
 /* Memento: A library to aid debugging of memory leaks/heap corruption.
  *
  * Usage:
  *    First, build your project with MEMENTO defined, and include this
- *    header file whereever you use malloc, realloc or free.
+ *    header file wherever you use malloc, realloc or free.
  *    This header file will use macros to point malloc, realloc and free to
  *    point to Memento_malloc, Memento_realloc, Memento_free.
  *
@@ -58,7 +61,7 @@
  *    the the code will wait until it reaches that event and then start
  *    checking the heap after every allocation event. Assuming it is a
  *    deterministic failure, you should then find out where in your program
- *    the error is occuring (between event x-1 and event x).
+ *    the error is occurring (between event x-1 and event x).
  *
  *    Then you can rerun the program again, and call
  *    Memento_breakAt(int event); and the program will call
@@ -134,7 +137,7 @@
 
 #ifndef MEMENTO_H
 
-#include "std.h"
+#include <memory.h>
 
 #define MEMENTO_H
 
@@ -169,14 +172,18 @@ int Memento_check(void);
 int Memento_setParanoia(int);
 int Memento_paranoidAt(int);
 int Memento_breakAt(int);
+void Memento_breakOnFree(void *a);
+void Memento_breakOnRealloc(void *a);
 int Memento_getBlockNum(void *);
 int Memento_find(void *a);
 void Memento_breakpoint(void);
 int Memento_failAt(int);
 int Memento_failThisEvent(void);
-
 void Memento_listBlocks(void);
 void Memento_listNewBlocks(void);
+size_t Memento_setMax(size_t);
+void Memento_stats(void);
+void *Memento_label(void *, const char *);
 
 void *Memento_malloc(size_t s);
 void *Memento_realloc(void *, size_t s);
@@ -199,19 +206,24 @@ void *Memento_calloc(size_t, size_t);
 #define Memento_realloc MEMENTO_UNDERLYING_REALLOC
 #define Memento_calloc  MEMENTO_UNDERLYING_CALLOC
 
-#define Memento_checkBlock(A)    0
-#define Memento_checkAllMemory() 0
-#define Memento_check()          0
-#define Memento_setParanoia(A)   0
-#define Memento_paranoidAt(A)    0
-#define Memento_breakAt(A)       0
-#define Memento_getBlockNum(A)   0
-#define Memento_find(A)          0
-#define Memento_breakpoint()     do {} while (0)
-#define Memento_failAt(A)        0
-#define Memento_failThisEvent()  0
-#define Memento_listBlocks()     do {} while (0)
-#define Memento_listNewBlocks()  do {} while (0)
+#define Memento_checkBlock(A)     0
+#define Memento_checkAllMemory()  0
+#define Memento_check()           0
+#define Memento_setParanoia(A)    0
+#define Memento_paranoidAt(A)     0
+#define Memento_breakAt(A)        0
+#define Memento_breakOnFree(A)    0
+#define Memento_breakOnRealloc(A) 0
+#define Memento_getBlockNum(A)    0
+#define Memento_find(A)           0
+#define Memento_breakpoint()      do {} while (0)
+#define Memento_failAt(A)         0
+#define Memento_failThisEvent()   0
+#define Memento_listBlocks()      do {} while (0)
+#define Memento_listNewBlocks()   do {} while (0)
+#define Memento_setMax(A)         0
+#define Memento_stats()           do {} while (0)
+#define Memento_label(A,B)        (A)
 
 #endif /* MEMENTO */
 
