@@ -22,6 +22,10 @@
 /* Ghostscript uses transitional LFS functions. */
 #define _LARGEFILE64_SOURCE 1
 
+#ifndef _FILE_OFFSET_BITS
+#define _FILE_OFFSET_BITS 64
+#endif
+
 /*
  * Here we deal with the vagaries of various C compilers.  We assume that:
  *      ANSI-standard Unix compilers define __STDC__.
@@ -357,16 +361,6 @@ typedef const char *ptr_ord_t;
 #define round_up(v, m) ROUND_UP(v, m)
 #define round_down(v, m) ROUND_DOWN(v, m)
 
-/*
- * In pre-ANSI C, float parameters get converted to double.
- * However, if we pass a float to a function that has been declared
- * with a prototype, and the parameter has been declared as float,
- * the ANSI standard specifies that the parameter is left as float.
- * To avoid problems caused by missing prototypes,
- * we declare almost all float parameters as double.
- */
-typedef double floatp;
-
 /*
  * Because of C's strange insistence that ; is a terminator and not a
  * separator, compound statements {...} are not syntactically equivalent to
