@@ -37,6 +37,10 @@
 #  ifdef MEMORY__NEED_MEMMOVE
 #    define memmove(dest,src,len) gs_memmove(dest,src,len)
 #  endif
+#  ifdef _MSC_VER
+#    define strcasecmp stricmp		/* MSC doesn't have the POSIX functions */
+#    define strncasecmp strnicmp
+#  endif
 #endif
 
 #endif /* string__INCLUDED */
