@@ -81,7 +81,7 @@ rpdl_print_page_copies(gx_device_printer * pdev, FILE * prn_stream, int num_coip
     if (pdev->PageCount == 0)
         rpdl_printer_initialize(pdev, prn_stream, num_coipes);
 
-    if (!(lprn->CompBuf = gs_malloc(gs_lib_ctx_get_non_gc_memory_t(), bpl * 3 / 2 + 1, maxY, "rpdl_print_page_copies(CompBuf)")))
+    if (!(lprn->CompBuf = gs_malloc(pdev->memory->non_gc_memory, bpl * 3 / 2 + 1, maxY, "rpdl_print_page_copies(CompBuf)")))
         return_error(gs_error_VMerror);
 
     lprn->NegativePrint = false; /* Not Support */
@@ -90,7 +90,7 @@ rpdl_print_page_copies(gx_device_printer * pdev, FILE * prn_stream, int num_coip
     if (code < 0)
         return code;
 
-    gs_free(gs_lib_ctx_get_non_gc_memory_t(), lprn->CompBuf, bpl * 3 / 2 + 1, maxY, "rpdl_print_page_copies(CompBuf)");
+    gs_free(pdev->memory->non_gc_memory, lprn->CompBuf, bpl * 3 / 2 + 1, maxY, "rpdl_print_page_copies(CompBuf)");
 
     fprintf(prn_stream, "\014");	/* Form  Feed */
 
