@@ -1,18 +1,20 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
+
 /* idisp.c */
-/* $Id$ */
 
 /*
  * This allows the interpreter to set the callback structure
@@ -94,7 +96,7 @@ display_set_callback(gs_main_instance *minst, display_callback *callback)
         if (was_open) {
             code = gs_opendevice(dev);
             if (code < 0) {
-                dprintf("**** Unable to open the display device, quitting.\n");
+                dmprintf(dev->memory, "**** Unable to open the display device, quitting.\n");
                 return_error(code);
             }
         }
