@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Font creation utilities */
 #include "memory_.h"
 #include "string_.h"
@@ -33,7 +35,7 @@
 #include "ipacked.h"
 #include "istruct.h"
 #include "store.h"
-
+#include "gsstate.h" /* for gs_currentcpsimode() */
 /* Structure descriptor and GC procedures for font_data */
 public_st_font_data();
 static
@@ -107,7 +109,7 @@ zfont_encode_char(gs_font *pfont, gs_char chr, gs_glyph_space_t gspace)
 
             if (gspace == GLYPH_SPACE_NOGEN)
                 return gs_no_glyph;
-            sprintf(buf, "j%ld", chr); /* 'j' is arbutrary. */
+            gs_sprintf(buf, "j%ld", chr); /* 'j' is arbutrary. */
             code = name_ref(pfont->memory, (const byte *)buf, strlen(buf), &tname, 1);
             if (code < 0) {
                 /* Can't propagate the error due to interface limitation,
@@ -129,7 +131,7 @@ zfont_glyph_name(gs_font *font, gs_glyph index, gs_const_string *pstr)
         char cid_name[sizeof(gs_glyph) * 3 + 1];
         int code;
 
-        sprintf(cid_name, "%lu", (ulong) index);
+        gs_sprintf(cid_name, "%lu", (ulong) index);
         code = name_ref(font->memory, (const byte *)cid_name, strlen(cid_name),
                         &nref, 1);
         if (code < 0)
@@ -673,6 +675,7 @@ build_gs_font(i_ctx_t *i_ctx_p, os_ptr op, gs_font ** ppfont, font_type ftype,
     gs_font *pfont;
     ref *pfid;
     ref *aop = dict_access_ref(op);
+    bool cpsi_mode = gs_currentcpsimode(imemory);
 
     get_font_name(imemory, &kname, op - 1);
     if (dict_find_string(op, "FontType", &pftype) <= 0 ||
@@ -691,11 +694,67 @@ build_gs_font(i_ctx_t *i_ctx_p, os_ptr op, gs_font ** ppfont, font_type ftype,
     if (pencoding) {   /* observed Adobe behavior */
         int count = r_size(pencoding);
         int type = ftype ? t_name : t_integer;
+        bool fixit = false;
+
         while (count--) {
            ref r;
-           if (array_get(imemory, pencoding, count, &r) < 0 ||
-             !(r_has_type(&r, type) || r_has_type(&r, t_null)))
-               return_error(e_typecheck);
+           if ((code = array_get(imemory, pencoding, count, &r)) < 0 ||
+             !(r_has_type(&r, type) || r_has_type(&r, t_null))) {
+               if (!cpsi_mode && ftype == ft_user_defined) {
+                   if (code < 0 || r_has_type(&r, t_null)) {
+                       return_error(e_typecheck);
+                   }
+                   fixit = true;
+                   break;
+               }
+               else {
+                   return_error(e_typecheck);
+               }
+           }
+        }
+
+        /* For at least Type 3 fonts, Adobe Distiller will "fix" an Encoding array, as in, for example
+         * Bug 692681 where the arrays contain integers rather than names. Once the font is instantiated
+         * the integers have been converted to names.
+         * It is preferable to to this manipulation here, rather than in Postscript, because we are less
+         * restricted by read-only attributes and VM save levels.
+         */
+        if (fixit) {
+            ref penc;
+            uint size = 0;
+            char buf[32], *bptr;
+            avm_space curglob = ialloc_space(idmemory);
+            avm_space useglob = r_is_local(pencoding) ? avm_local : avm_global;
+
+            ialloc_set_space(idmemory, useglob);
+            
+            count = r_size(pencoding);
+            if ((code = ialloc_ref_array(&penc, (r_type_attrs(pencoding) & a_readonly), count, "build_gs_font")) < 0)
+                 return code;
+            
+            while (count--) {
+               ref r;
+               if (array_get(imemory, pencoding, count, &r) < 0){
+                   return_error(e_typecheck);
+               }
+               /* For type 3, we know the Encoding entries must be names */
+               if (r_has_type(&r, t_name)){
+                   ref_assign(&(penc.value.refs[count]), &r);
+               }
+               else {
+               
+                   if ((code = obj_cvs(imemory, &r, (byte *)buf, 32, &size, (const byte **)(&bptr))) < 0) {
+                       return(code);
+                   }
+                   if ((code = name_ref(imemory, (const byte *)bptr, size, &r, true)) < 0)
+                        return code;
+                   ref_assign(&(penc.value.refs[count]), &r);
+               }
+            }
+            
+            if ((code = dict_put_string(osp, "Encoding", &penc, NULL)) < 0)
+               return code;
+            ialloc_set_space(idmemory, curglob);
         }
     }
     if ((code = dict_int_param(op, "WMode", 0, 1, 0, &wmode)) < 0 ||
